----------------------------------------------------------------
-- Provides some auxiliary functions, mainly on strings,      --
-- that are used on several occasions throughout the program. --
--                                                            --
--- Finished refactoring: 16-sep-2010                         --
----------------------------------------------------------------

module Auxiliary where

import Debug.Trace

-- Note: because of this visualisation of expression,
-- the output of the LPPE is not parsable again.
type Variable     = String
type Function     = String
data Expression = Variable Variable | Function Function [Expression] deriving Eq 
instance Show Expression where
  show (Variable variable)   = variable
  show (Function "eq" [x,y]) = show x ++ " = " ++ show y
  show (Function "not" [(Function "eq" [x,y])]) = show x ++ " != " ++ show y
  show (Function "and" pars) = infixString (map show pars) " & "
  show (Function name pars)  = name ++ optionalParentheses (infixString (map show pars) ", ")


-- These functions allow easy manipulation of triples.
fst3 (a,b,c) = a
snd3 (a,b,c) = b
thd3 (a,b,c) = c

-- This function takes a list of strings and a separating string
-- and produces one string that consists of all the strings of the list
-- in a row, separated by the separating string.
infixString :: [String] -> String -> String
infixString []     inf = []
infixString [x]    inf = x
infixString (x:xs) inf = x ++ inf ++ (infixString xs inf)

-- This function takes a string and surrounds it by parentheses,
-- except for the case that it is empty.
optionalParentheses :: String -> String
optionalParentheses ""   = ""
optionalParentheses text = "(" ++ text ++ ")"  

-- This function takes a list of pairs of strings [(s1,t1), (s2,t2), ...], 
-- and provides the string (s1 : t1, s2 : t2, ...)
printPairs :: [(String, String)] -> String
printPairs pairs = optionalParentheses (infixString [left ++ " : " ++ right | (left, right) <- pairs] ", ")

-- This function takes a list of items and makes a comma-separated list of it,
-- surrounded by parenthesis.
commaList :: [String] -> String
commaList items = optionalParentheses (infixString items ", ")

-- The function provides the string between a certain begin and end character,
-- and also the remaining string after the end character.
takeBetween :: String -> Char -> Char -> (String, String)
takeBetween s from to = (between, remainder)
  where
    between   = takeWhile (/= to) (drop 1 (dropWhile (/= from) s))
    remainder = drop 1 (dropWhile (/= to) (drop 1 (dropWhile (/= from) s)))
	
-- This function takes a string and a delimiter, and splits the string
-- based on this delimiter.
split :: String -> Char -> [String]
split []     delim              = [""]
split (c:cs) delim | c == delim = "" : rest
                   | otherwise  = (c : head rest) : tail rest
  where
    rest = split cs delim

-- This functions removes the spaces from a string
removeSpaces ""      = ""
removeSpaces (' ': s) = removeSpaces s
removeSpaces (x  :xs)  = x:(removeSpaces xs)

-- This function changes the line breaks of a string to spaces.
removeLineBreaks :: String -> String
removeLineBreaks []                   = []
removeLineBreaks (x:xs) | trace("blaat") ('\n' ==) x = ' ' : removeLineBreaks xs
                        | ('\r' ==) x = ' ' : removeLineBreaks xs
						| otherwise   =  x  : removeLineBreaks xs