module FromPRISM where

import PrismParser
import Auxiliary
import LPPE
import Expressions
import Debug.Trace

fromPrism :: String -> String
fromPrism input = prismToPRCRL (prismParse (prismLexer (removeLineBreaks (removeComments input))))

removeComments ""             = ""
removeComments ('/':'/':rest) = removeComments (dropWhile (/= '\n') rest)
removeComments (c:cs)         = c:(removeComments cs)


prismToPRCRL :: PrismModel -> String
prismToPRCRL (PrismModel globals mods) = "spec: \n\n   " ++ (infixString (map printModule newMods) " \n\n   ") ++ "\n\n  " ++ (printInits newMods)
  where
    newMods = map (addGlobals globals) mods

addGlobals :: Globals -> Module -> Module
addGlobals globals (Module name variables commands) = Module name (variables ++ globals) commands


printInits mods = " init " ++ (infixString (map printModInits  mods) " || " )

printModInits (Module name variables commands) = name ++ vars
  where
    vars = commaList [initialValue prismvariable | prismvariable <- variables]

initialValue (IntRange (var,from,to,initial)) = show initial
initialValue (Boolean (var,initial)) | initial = "T"
                                     | otherwise = "F"


printModule (Module name variables commands) = name ++ pars ++ " = \n      " ++ summands
  where
    pars     = commaList ([printVariableDef varDef | varDef <- variables]  )
    summands = infixString (map (printCommand name) commands) " \n    + " 

printVariableDef :: PRISMVariable -> String
printVariableDef (IntRange (var,from,to,initial)) = var ++ " : {" ++ show from ++ ".." ++ show to ++ "}"
printVariableDef (Boolean (var,initial))          = var ++ " : Bool"

printCommand name (cond, act, nextstate) | length nextstate == 1 = printExpression cond ++ " => " ++ act ++ "." ++ name ++ (commaList (map printUpdate (snd (nextstate!!0)))) 
                                    | otherwise             = printExpression cond ++ " => " ++ act ++ ".psum(" ++ infixString (map (printNextstate name) nextstate) " + " ++ ")"

printNextstate name (prob, updates) = printExpression prob ++ " -> " ++ name ++ (commaList (map printUpdate updates))

printUpdate (var, expr) = var ++ " := " ++ printExpression expr