---------------------------------------------------------
-- Takes a string containing a prCRL specification and --
-- provides the system is describes.                   --
--                                                     --
-- Finished refactoring: 16-sep-2010                   --
-- Nog weer verbeteren! 
---------------------------------------------------------

module InputParser where

import LPPE
import Expressions
import Processes
import Linearise
import Auxiliary
import Parser
import DataSpec
import ToPA
import List
import Debug.Trace

type Constants = [(String, Expression)]

-----------------------
-- Parsing the input --
-----------------------

-- This function takes a prCRL specification, and returns the corresponding
-- LPPE including its initial state.
parseInput :: Bool -> Constants -> String -> (Specification, [(String, Type)], [String])
parseInput prismComposition constants1 input | correctSpecification = ((fst system2, snd system2, newDataspec), actiontypes, untilformula)
                                             | otherwise            = error("Error in specification.")
  where
    input2                 = input -- removeLineBreaks input
    (processes, initials,hiding,encapsulation, renaming, communication, constants, datatypes, functions, actiontypes, untilformula)  = parseProcesses constants1 dataspec input2
    dataspec               = (datatypes ++ builtInTypes, [], functions ++ builtInFunctions)
    system_                = fst (makeLPPE 1 prismComposition communication constants dataspec processes initials)
    system                 = if (prismComposition) then fixGlobalVariables system_ else system_
    system2                = rename (hide (encapsulate system encapsulation) hiding) renaming
    summands               = getSummands (fst system2)
    pars                   = getLPPEPars (fst system2)
    types                  = nub ([TypeName "Bool"] 
                             ++ allTypes (fst3 dataspec)                            
                             ++ map snd pars ++ [t | summand <- summands, (v,t) <- getLocalPars summand]
                             ++ [t | summand <- summands, (v,t,f) <- getProbChoices summand] )
    typeValues             = [(t, allValues t (fst3 dataspec)) | t <- types]
    newDataspec            = (fst3 dataspec, typeValues, thd3 dataspec)
    correctSpecification   = distinctProcesses processes &&
                             noUnguardedRecursion processes processes &&
                             validateData system2 newDataspec && 
                             correctInits system2 newDataspec

distinctProcesses :: [Process] -> Bool
distinctProcesses procs | correct      = True
                        | not(correct) = error("Error: Not all processes have a distinct name, namely: " ++ show erroneous)
  where
    correct   = nub (map getProcName procs) == map getProcName procs
    erroneous = (map getProcName procs) \\(nub (map getProcName procs))

noUnguardedRecursion :: [Process] -> [Process] -> Bool
noUnguardedRecursion allProcs []                          = True
noUnguardedRecursion allProcs (p:ps) | unguardedRecursion = error("Error: Unguarded recursion of process " ++ name)
                                     | otherwise          = noUnguardedRecursion allProcs ps
  where
    name               = getProcName p
    unguardedRecursion = elem name (reachableUnguarded allProcs [] (getRHS p))

reachableUnguarded :: [Process] -> [String] -> ProcessTerm -> [String]
reachableUnguarded allProcs seen (ActionPrefix _ _ _ rhs)  = []
reachableUnguarded allProcs seen (Sum _ _ rhs)             = reachableUnguarded allProcs seen rhs
reachableUnguarded allProcs seen (Implication _ rhs)       = reachableUnguarded allProcs seen rhs
reachableUnguarded allProcs seen (ActionPrefix2 _ _ nexts) = []
reachableUnguarded allProcs seen (Plus rhss)               = concat (map (reachableUnguarded allProcs seen) rhss)
reachableUnguarded allProcs seen (ProcessInstance name _)    
  | elem name seen = []
  | otherwise      = name:(reachableUnguarded allProcs (name:seen) (getRHS (resolveProcess name allProcs))) 


validateData :: System -> DataSpec -> Bool
validateData (lppe, init) (datatypes, _, functions) = valid
  where
    usedTypes = findTypesInLPPE lppe
    valid     = and (map (checkType datatypes) usedTypes)

correctInits :: System -> DataSpec -> Bool
correctInits (lppe, init) dataspec | not enoughInitials = error("Error: Number of initial values not correct.")
                                   | result    = True
                                   | otherwise = error("Error: Initial value incorrect. Input " ++ fst (initials!!index) ++ " is not of type " ++ printType (snd (initials!!index))) 
  where
    initials = zip init (map snd (getLPPEPars lppe))
    elements = [elem var (getValues dataspec typ) | (var, typ) <- initials]
    enoughInitials = length init == length (getLPPEPars lppe) 
    result   = and elements 
    index    = [i | i <- [0..length elements - 1], elements!!i == False]!!0

checkType :: [DataType] -> Type -> Bool
checkType _ (TypeRange from to) = True
checkType [] (TypeName name) = error("Error: Undefined type " ++ name ++ " used.")
checkType ((EnumInt name from to):xs) (TypeName t) | t == name = True
                                        | otherwise = checkType xs (TypeName t)
checkType ((EnumString name values):xs) (TypeName t) | t == name = True
                                           | otherwise = checkType xs (TypeName t)
 
findTypesInLPPE (LPPE name params summands) = map snd params


parseProcesses :: Constants -> DataSpec -> String -> ([Process], InitialProcessDefinition, [String], [String], [(String, String)], CommFunction, Constants, [DataType], [FunctionDef], [(String, Type)], [String])
parseProcesses commandlineConstants dataspec processesString | correctParse && correctInput = (processes5, initialProcesses, hiding, encapsulation, renaming, commFunction, constants, types, functions, actiontypes, untilformula)
                                                             | otherwise                    = error(errorMessage)
  where
    parsedInput_                 = parse processesString 1
    correctParse                 = case parsedInput_ of Ok a -> True 
                                                        Failed s -> False
    (Failed errorMessage)        = parsedInput_
    (Ok parsedInput)             = parsedInput_ 
    processes1                   = getProcesses parsedInput
    initialProcessDefs           = getInitialProcesses parsedInput
    initialProcesses             = head initialProcessDefs
    datapart                     = parsedInput
    processes2                   = processes1 
    processes3                   = map (unfoldUpdates processes2) processes2 
    processes4                   = functionaliseContext 1 processes3
    functions                    = getFunctions datapartnew
    processes5                   = map (substituteConstantsInProcess functions constants) processes4
    constants                    = commandlineConstants ++ getConstants functions datapart
    datapartnew                  = applyConstants datapart constants
    hiding                       = getHiddenActions datapartnew
    renaming                     = getRenamedActions datapartnew
    encapsulation                = getEncapsulatedActions datapartnew
    commFunction                 = \x -> \y -> createCommFunction x y (getCommunications datapartnew)
    types                        = getTypes functions datapartnew
    untilformula                 = getUntilFormula datapartnew
    actiontypes                  = getActionTypes functions datapartnew
    correctInput                 = checkInput initialProcessDefs

checkInput initialProcesses | length initialProcesses == 0 = error("Error: no initial process defined.")
                            | length initialProcesses > 1  = error("Error: more than one initial process defined.")
                            | otherwise                    = True

applyConstants [] constants = []
applyConstants ((DataRangeType name from to):rest) constants = (DataRangeType name newFrom newTo):(applyConstants rest constants)
  where
    newFrom = substituteInExpression constants from
    newTo   = substituteInExpression constants to
applyConstants (x:rest) constants = x:(applyConstants rest constants)

getProcesses []                            = []
getProcesses ((ParserProcess proc):rest)   = proc:(getProcesses rest)
getProcesses (_:rest)                    = getProcesses rest

getInitialProcesses []           = []
getInitialProcesses ((ParserInitialProcess inits):rest) = inits:(getInitialProcesses rest)
getInitialProcesses (_:rest)                 = getInitialProcesses rest

getHiddenActions []                          = []
getHiddenActions ((DataHiding actions):rest) = actions ++ getHiddenActions rest
getHiddenActions (_:rest)                    = getHiddenActions rest

getRenamedActions []                                = []
getRenamedActions ((DataRenaming actionpairs):rest) = actionpairs ++ getRenamedActions rest
getRenamedActions (_:rest)                          = getRenamedActions rest

getUntilFormula []                                = []
getUntilFormula ((DataUntilFormula formula):rest) = formula:(getUntilFormula rest)
getUntilFormula (_:rest)                          = getUntilFormula rest

getEncapsulatedActions []                                 = []
getEncapsulatedActions ((DataEncapsulation actions):rest) = actions ++ getEncapsulatedActions rest
getEncapsulatedActions (_:rest)                           = getEncapsulatedActions rest

getCommunications []                               = []
getCommunications ((DataCommunication comms):rest) = comms ++ getCommunications rest
getCommunications (_:rest)                         = getCommunications rest

getConstants _ []                               = []
getConstants functions ((DataConstant name value):rest) = (name, Variable (evalExpr ([], [], functions) [] value)):(getConstants functions rest)
getConstants functions (_:rest)                         = getConstants functions rest

getActionTypes :: [FunctionDef] -> [Item] -> [(Action, Type)]
getActionTypes functions []                           = []
getActionTypes functions ((DataActions actions):rest) = map (evaluateActionType functions) actions ++ (getActionTypes functions rest)
getActionTypes functions (_:rest)                     = getActionTypes functions rest

evaluateActionType functions (name, None)                       = (name, NoType)
evaluateActionType functions (name, Bool)                       = (name, TypeName "Bool")
evaluateActionType functions (name, IntRange from to) | correct = (name, TypeRange (read fromInt) (read toInt))
                                                      | otherwise = error("Error: could not parse the action type " ++ name ++ " = {" ++ show from ++ ".." ++ show to ++ "}")
  where
    fromInt = evalExpr ([], [], functions) [] from
    toInt   = evalExpr ([], [], functions) [] to
    correct = isInteger fromInt && isInteger toInt


getTypes functions []                                                   = []
getTypes functions ((DataEnumType name values):rest)   | name /= "Bool" = (EnumString name values):(getTypes functions rest)
getTypes functions ((DataRangeType name from to):rest)                  | correct = (EnumInt name (read fromValue) (read toValue)):(getTypes functions rest)
                                                              | otherwise = error("Error: could not parse the type " ++ name ++ " = {" ++ show from ++ ".." ++ show to ++ "}")
  where
    fromValue = evalExpr ([],[],functions) [] from
    toValue   = evalExpr ([],[],functions) [] to
    correct   = isInteger fromValue && isInteger toValue 
getTypes functions (_:rest)                                             = getTypes functions rest

getFunctions []                                 = []
getFunctions ((DataFunction name mapping):rest) = (FunctionDef name mapping):(getFunctions rest)
getFunctions (_:rest)                           = getFunctions rest


functionaliseContext i [] = []
functionaliseContext i (r:rs) = (functionaliseProbabilities i r):(functionaliseContext (i+1) rs)

makeLPPE :: Int -> Bool -> CommFunction -> Constants -> DataSpec -> [Process] -> InitialProcessDefinition -> (System, Int)
makeLPPE nr prismComposition communication constants dataspec processes (InitSingleProcess proc)   = (system, nr + 1)
  where
    (lppe, init) = processToLPPE constants dataspec processes proc
    system = (suffixStringLPPE ("_" ++ show nr) lppe, init)
makeLPPE nr prismComposition communication constants dataspec processes (InitParallel proc1 proc2) = (composeSystems prismComposition system1 system2 communication, nr2)
  where
    (system1,nr1) = makeLPPE nr prismComposition communication constants dataspec processes proc1
    (system2, nr2) = makeLPPE nr1 prismComposition communication constants dataspec processes proc2
makeLPPE nr prismComposition communication constants dataspec processes (InitHiding actions proc) = (hide system actions, nr2)
  where
    (system,nr2) = makeLPPE nr prismComposition communication constants dataspec processes proc	
makeLPPE nr prismComposition communication constants dataspec processes (InitEncapsulation actions proc) = (encapsulate system actions, nr2)
  where
    (system,nr2) = makeLPPE nr prismComposition communication constants dataspec processes proc	
makeLPPE nr prismComposition communication constants dataspec processes (InitRenaming actions proc) = (rename system actions, nr2)
  where
    (system,nr2) = makeLPPE nr prismComposition communication constants dataspec processes proc	

processToLPPE :: Constants -> DataSpec -> [Process] -> InitialProcess -> System
processToLPPE constants dataspec processes (InitialProcess name initials) = system
  where
    Process procname params rhs = resolveProcess name processes
    newProcesses                = (Process procname params rhs):[p | p <- processes, getProcName p /= name]
    initialProcess              = [i | i <- [0..length processes - 1], getProcName (processes!!i) == name]!!0
    newInitials                 =  map (\x -> if   elem x (map fst constants) 
                                              then ([val | (y, Variable val) <- constants, x == y]!!0)
   	                                          else x) (map (evalExpr dataspec []) initials)
    system                      = if (isLPPE procname (length params) rhs)    
	                              then (LPPE procname params (parseLPPE rhs), newInitials) 
	                              else (linearise dataspec newProcesses newInitials)



substituteConstantsInProcess :: [FunctionDef] -> Constants -> Process -> Process
substituteConstantsInProcess functions subs (Process name params rhs) = newProcess
  where
    newRHSs    = substituteConstantsInProcessTerm functions subs rhs
    newParams  = [(v,substituteConstantsInType functions subs t) | (v,t) <- params]
    newProcess = Process name newParams newRHSs

substituteConstantsInProcessTerm :: [FunctionDef] -> [(Variable, Expression)] -> ProcessTerm -> ProcessTerm
substituteConstantsInProcessTerm functions subs (Plus rs)           = Plus (map (substituteConstantsInProcessTerm functions subs) rs)
substituteConstantsInProcessTerm functions subs (Implication c rhs) = Implication (substituteInExpression subs c) 
                                                                        (substituteConstantsInProcessTerm functions subs rhs)
substituteConstantsInProcessTerm functions subs (Sum var typ rhs)   = Sum var typNew (substituteConstantsInProcessTerm functions subs rhs)
  where
    typNew = substituteConstantsInType functions subs typ
substituteConstantsInProcessTerm functions subs (ActionPrefix a aps probs rhs) = ActionPrefix a newAps newProbs (substituteConstantsInProcessTerm functions subs rhs)
    where
      newAps   = map (substituteInExpression subs) aps
      newProbs = [(v, substituteConstantsInType functions subs t, substituteInExpression subs f) | (v,t,f) <- probs]
substituteConstantsInProcessTerm functions subs (ProcessInstance name pars) = ProcessInstance name (map (substituteInExpression subs) pars)

substituteConstantsInType :: [FunctionDef] -> [Substitution] -> Type -> Type
substituteConstantsInType functions subs (TypeName name)                = TypeName name
substituteConstantsInType functions subs (TypeRange from to)            = TypeRange from to
substituteConstantsInType functions subs (TypeRangeExpressions from to) | correct   = TypeRange (read newFrom) (read newTo)
                                                                        | otherwise = error("Error: could not parse type {" ++ show from ++ ".." ++ show to ++ "}")
  where
    newFrom = evalExpr ([],[],functions) [] (substituteInExpression subs from) 
    newTo   = evalExpr ([],[],functions) [] (substituteInExpression subs to)
    correct = isInteger newFrom && isInteger newTo
