{-# OPTIONS_GHC -fno-warn-overlapping-patterns #-}
module Parser where

import Char
import Processes
import Expressions
import Auxiliary
import DataSpec
import Debug.Trace

data Item = DataActions [(String, ActionType)]
          | DataHiding [String] 
          | DataRenaming [(String, String)] 
          | DataUntilFormula String
          | DataEncapsulation [String]   
          | DataCommunication [(String, String, String)] 
          | DataConstant String Expression
          | DataEnumType String [String] 
          | DataRangeType String Expression Expression
          | DataFunction String [([String], String)]
          | ParserProcess Process
          | ParserInitialProcess InitialProcessDefinition
              deriving (Show, Eq)

data ActionType = None | Bool | IntRange Expression Expression deriving (Show, Eq)

data InitialProcessDefinition = InitSingleProcess InitialProcess
                              | InitParallel InitialProcessDefinition InitialProcessDefinition
                              | InitHiding [Action] InitialProcessDefinition
                              | InitEncapsulation [Action] InitialProcessDefinition
                              | InitRenaming [(Action, Action)] InitialProcessDefinition
                                   deriving (Show, Eq)

type LineNumber = Int
data ParseResult a = Ok a | Failed String
type ParserMonad a = String -> LineNumber -> ParseResult a

getLineNo :: ParserMonad LineNumber
getLineNo = \s l -> Ok l


thenParserMonad :: ParserMonad a -> (a -> ParserMonad b) -> ParserMonad b
m `thenParserMonad ` k = \s -> \l ->
   case (m s l) of 
       Ok a -> k a s l
       Failed e -> Failed e

returnParserMonad :: a -> ParserMonad a
returnParserMonad a = \s -> \l -> Ok a

failParserMonad :: String -> ParserMonad a
failParserMonad err = \s -> \l -> Failed (err ++ " on line " ++ show l)

-- parser produced by Happy Version 1.18.5

data HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 t21 t22 t23 t24 t25 t26 t27 t28 t29 t30 t31 t32
	= HappyTerminal (Token)
	| HappyErrorToken Int
	| HappyAbsSyn4 t4
	| HappyAbsSyn5 t5
	| HappyAbsSyn6 t6
	| HappyAbsSyn7 t7
	| HappyAbsSyn8 t8
	| HappyAbsSyn9 t9
	| HappyAbsSyn10 t10
	| HappyAbsSyn11 t11
	| HappyAbsSyn12 t12
	| HappyAbsSyn13 t13
	| HappyAbsSyn14 t14
	| HappyAbsSyn15 t15
	| HappyAbsSyn16 t16
	| HappyAbsSyn17 t17
	| HappyAbsSyn18 t18
	| HappyAbsSyn19 t19
	| HappyAbsSyn20 t20
	| HappyAbsSyn21 t21
	| HappyAbsSyn22 t22
	| HappyAbsSyn23 t23
	| HappyAbsSyn24 t24
	| HappyAbsSyn25 t25
	| HappyAbsSyn26 t26
	| HappyAbsSyn27 t27
	| HappyAbsSyn28 t28
	| HappyAbsSyn29 t29
	| HappyAbsSyn30 t30
	| HappyAbsSyn31 t31
	| HappyAbsSyn32 t32

action_0 (59) = happyShift action_4
action_0 (64) = happyShift action_5
action_0 (65) = happyShift action_6
action_0 (66) = happyShift action_7
action_0 (68) = happyShift action_8
action_0 (69) = happyShift action_9
action_0 (70) = happyShift action_10
action_0 (71) = happyShift action_11
action_0 (72) = happyShift action_12
action_0 (73) = happyShift action_13
action_0 (74) = happyShift action_14
action_0 (4) = happyGoto action_15
action_0 (5) = happyGoto action_2
action_0 (6) = happyGoto action_3
action_0 _ = happyReduce_2

action_1 (59) = happyShift action_4
action_1 (64) = happyShift action_5
action_1 (65) = happyShift action_6
action_1 (66) = happyShift action_7
action_1 (68) = happyShift action_8
action_1 (69) = happyShift action_9
action_1 (70) = happyShift action_10
action_1 (71) = happyShift action_11
action_1 (72) = happyShift action_12
action_1 (73) = happyShift action_13
action_1 (74) = happyShift action_14
action_1 (5) = happyGoto action_2
action_1 (6) = happyGoto action_3
action_1 _ = happyFail

action_2 _ = happyReduce_1

action_3 (59) = happyShift action_4
action_3 (64) = happyShift action_5
action_3 (65) = happyShift action_6
action_3 (66) = happyShift action_7
action_3 (68) = happyShift action_8
action_3 (69) = happyShift action_9
action_3 (70) = happyShift action_10
action_3 (71) = happyShift action_11
action_3 (72) = happyShift action_12
action_3 (73) = happyShift action_13
action_3 (74) = happyShift action_14
action_3 (5) = happyGoto action_40
action_3 (6) = happyGoto action_3
action_3 _ = happyReduce_2

action_4 (40) = happyShift action_35
action_4 (64) = happyShift action_36
action_4 (65) = happyShift action_37
action_4 (66) = happyShift action_38
action_4 (69) = happyShift action_39
action_4 (18) = happyGoto action_34
action_4 _ = happyFail

action_5 (40) = happyShift action_32
action_5 (61) = happyShift action_33
action_5 _ = happyFail

action_6 (64) = happyShift action_24
action_6 (17) = happyGoto action_31
action_6 _ = happyFail

action_7 (40) = happyShift action_30
action_7 (15) = happyGoto action_28
action_7 (16) = happyGoto action_29
action_7 _ = happyFail

action_8 (64) = happyShift action_27
action_8 (8) = happyGoto action_25
action_8 (9) = happyGoto action_26
action_8 _ = happyFail

action_9 (64) = happyShift action_24
action_9 (17) = happyGoto action_23
action_9 _ = happyFail

action_10 (40) = happyShift action_22
action_10 (13) = happyGoto action_20
action_10 (14) = happyGoto action_21
action_10 _ = happyFail

action_11 (64) = happyShift action_19
action_11 _ = happyFail

action_12 (38) = happyShift action_18
action_12 _ = happyFail

action_13 (64) = happyShift action_17
action_13 _ = happyFail

action_14 (64) = happyShift action_16
action_14 _ = happyFail

action_15 (75) = happyAccept
action_15 _ = happyFail

action_16 (61) = happyShift action_75
action_16 _ = happyFail

action_17 (61) = happyShift action_74
action_17 _ = happyFail

action_18 (35) = happyShift action_68
action_18 (40) = happyShift action_69
action_18 (41) = happyShift action_70
action_18 (44) = happyShift action_71
action_18 (48) = happyShift action_72
action_18 (64) = happyShift action_73
action_18 (7) = happyGoto action_67
action_18 _ = happyReduce_17

action_19 (61) = happyShift action_66
action_19 _ = happyFail

action_20 _ = happyReduce_8

action_21 (35) = happyShift action_65
action_21 _ = happyReduce_34

action_22 (64) = happyShift action_64
action_22 _ = happyFail

action_23 _ = happyReduce_7

action_24 (35) = happyShift action_63
action_24 _ = happyReduce_40

action_25 _ = happyReduce_6

action_26 (35) = happyShift action_62
action_26 _ = happyReduce_24

action_27 (53) = happyShift action_61
action_27 _ = happyReduce_26

action_28 _ = happyReduce_5

action_29 (35) = happyShift action_60
action_29 _ = happyReduce_37

action_30 (64) = happyShift action_59
action_30 _ = happyFail

action_31 _ = happyReduce_4

action_32 (64) = happyShift action_58
action_32 (26) = happyGoto action_56
action_32 (27) = happyGoto action_57
action_32 _ = happyFail

action_33 (33) = happyShift action_51
action_33 (40) = happyShift action_52
action_33 (48) = happyShift action_53
action_33 (50) = happyShift action_54
action_33 (64) = happyShift action_55
action_33 (21) = happyGoto action_48
action_33 (22) = happyGoto action_49
action_33 (31) = happyGoto action_50
action_33 _ = happyFail

action_34 (37) = happyShift action_47
action_34 _ = happyReduce_14

action_35 (40) = happyShift action_35
action_35 (64) = happyShift action_36
action_35 (65) = happyShift action_37
action_35 (66) = happyShift action_38
action_35 (69) = happyShift action_39
action_35 (18) = happyGoto action_46
action_35 _ = happyFail

action_36 (38) = happyShift action_45
action_36 (20) = happyGoto action_44
action_36 _ = happyReduce_51

action_37 (40) = happyShift action_43
action_37 _ = happyFail

action_38 (40) = happyShift action_42
action_38 _ = happyFail

action_39 (40) = happyShift action_41
action_39 _ = happyFail

action_40 _ = happyReduce_3

action_41 (64) = happyShift action_24
action_41 (17) = happyGoto action_129
action_41 _ = happyFail

action_42 (40) = happyShift action_30
action_42 (15) = happyGoto action_128
action_42 (16) = happyGoto action_29
action_42 _ = happyFail

action_43 (64) = happyShift action_24
action_43 (17) = happyGoto action_127
action_43 _ = happyFail

action_44 _ = happyReduce_42

action_45 (39) = happyShift action_126
action_45 (40) = happyShift action_87
action_45 (48) = happyShift action_53
action_45 (50) = happyShift action_54
action_45 (64) = happyShift action_88
action_45 (30) = happyGoto action_124
action_45 (31) = happyGoto action_125
action_45 _ = happyFail

action_46 (37) = happyShift action_47
action_46 (44) = happyShift action_123
action_46 _ = happyFail

action_47 (40) = happyShift action_35
action_47 (64) = happyShift action_36
action_47 (65) = happyShift action_37
action_47 (66) = happyShift action_38
action_47 (69) = happyShift action_39
action_47 (18) = happyGoto action_122
action_47 _ = happyFail

action_48 (46) = happyShift action_121
action_48 _ = happyReduce_16

action_49 _ = happyReduce_60

action_50 (41) = happyShift action_108
action_50 (45) = happyShift action_109
action_50 (47) = happyShift action_110
action_50 (49) = happyShift action_111
action_50 (50) = happyShift action_112
action_50 (51) = happyShift action_113
action_50 (52) = happyShift action_114
action_50 (55) = happyShift action_115
action_50 (56) = happyShift action_116
action_50 (58) = happyShift action_117
action_50 (60) = happyShift action_118
action_50 (61) = happyShift action_119
action_50 (62) = happyShift action_120
action_50 _ = happyFail

action_51 (40) = happyShift action_107
action_51 _ = happyFail

action_52 (33) = happyShift action_51
action_52 (40) = happyShift action_52
action_52 (48) = happyShift action_53
action_52 (50) = happyShift action_54
action_52 (64) = happyShift action_55
action_52 (21) = happyGoto action_105
action_52 (22) = happyGoto action_49
action_52 (31) = happyGoto action_106
action_52 _ = happyFail

action_53 (40) = happyShift action_87
action_53 (48) = happyShift action_53
action_53 (50) = happyShift action_54
action_53 (64) = happyShift action_88
action_53 (31) = happyGoto action_104
action_53 _ = happyFail

action_54 (40) = happyShift action_87
action_54 (48) = happyShift action_53
action_54 (50) = happyShift action_54
action_54 (64) = happyShift action_88
action_54 (31) = happyGoto action_103
action_54 _ = happyFail

action_55 (38) = happyShift action_101
action_55 (40) = happyShift action_102
action_55 (41) = happyReduce_93
action_55 (44) = happyReduce_93
action_55 (45) = happyReduce_93
action_55 (47) = happyReduce_93
action_55 (49) = happyReduce_93
action_55 (50) = happyReduce_93
action_55 (51) = happyReduce_93
action_55 (52) = happyReduce_93
action_55 (55) = happyReduce_93
action_55 (56) = happyReduce_93
action_55 (58) = happyReduce_93
action_55 (60) = happyReduce_93
action_55 (61) = happyReduce_93
action_55 (62) = happyReduce_93
action_55 (63) = happyReduce_93
action_55 (19) = happyGoto action_100
action_55 _ = happyReduce_48

action_56 (35) = happyShift action_98
action_56 (44) = happyShift action_99
action_56 _ = happyFail

action_57 _ = happyReduce_70

action_58 (53) = happyShift action_97
action_58 _ = happyFail

action_59 (35) = happyShift action_96
action_59 _ = happyFail

action_60 (40) = happyShift action_30
action_60 (15) = happyGoto action_95
action_60 (16) = happyGoto action_29
action_60 _ = happyFail

action_61 (42) = happyShift action_93
action_61 (67) = happyShift action_94
action_61 _ = happyFail

action_62 (64) = happyShift action_27
action_62 (8) = happyGoto action_92
action_62 (9) = happyGoto action_26
action_62 _ = happyFail

action_63 (64) = happyShift action_24
action_63 (17) = happyGoto action_91
action_63 _ = happyFail

action_64 (35) = happyShift action_90
action_64 _ = happyFail

action_65 (40) = happyShift action_22
action_65 (13) = happyGoto action_89
action_65 (14) = happyGoto action_21
action_65 _ = happyFail

action_66 (40) = happyShift action_87
action_66 (48) = happyShift action_53
action_66 (50) = happyShift action_54
action_66 (64) = happyShift action_88
action_66 (31) = happyGoto action_86
action_66 _ = happyFail

action_67 (39) = happyShift action_85
action_67 _ = happyFail

action_68 (35) = happyShift action_68
action_68 (40) = happyShift action_69
action_68 (41) = happyShift action_70
action_68 (44) = happyShift action_71
action_68 (48) = happyShift action_72
action_68 (64) = happyShift action_73
action_68 (7) = happyGoto action_84
action_68 _ = happyReduce_17

action_69 (35) = happyShift action_68
action_69 (40) = happyShift action_69
action_69 (41) = happyShift action_70
action_69 (44) = happyShift action_71
action_69 (48) = happyShift action_72
action_69 (64) = happyShift action_73
action_69 (7) = happyGoto action_83
action_69 _ = happyReduce_17

action_70 (35) = happyShift action_68
action_70 (40) = happyShift action_69
action_70 (41) = happyShift action_70
action_70 (44) = happyShift action_71
action_70 (48) = happyShift action_72
action_70 (64) = happyShift action_73
action_70 (7) = happyGoto action_82
action_70 _ = happyReduce_17

action_71 (35) = happyShift action_68
action_71 (40) = happyShift action_69
action_71 (41) = happyShift action_70
action_71 (44) = happyShift action_71
action_71 (48) = happyShift action_72
action_71 (64) = happyShift action_73
action_71 (7) = happyGoto action_81
action_71 _ = happyReduce_17

action_72 (35) = happyShift action_68
action_72 (40) = happyShift action_69
action_72 (41) = happyShift action_70
action_72 (44) = happyShift action_71
action_72 (48) = happyShift action_72
action_72 (64) = happyShift action_73
action_72 (7) = happyGoto action_80
action_72 _ = happyReduce_17

action_73 (35) = happyShift action_68
action_73 (40) = happyShift action_69
action_73 (41) = happyShift action_70
action_73 (44) = happyShift action_71
action_73 (48) = happyShift action_72
action_73 (64) = happyShift action_73
action_73 (7) = happyGoto action_79
action_73 _ = happyReduce_17

action_74 (40) = happyShift action_77
action_74 (42) = happyShift action_78
action_74 _ = happyFail

action_75 (40) = happyShift action_76
action_75 _ = happyFail

action_76 (64) = happyShift action_175
action_76 (10) = happyGoto action_172
action_76 (11) = happyGoto action_173
action_76 (12) = happyGoto action_174
action_76 _ = happyFail

action_77 (64) = happyShift action_24
action_77 (17) = happyGoto action_171
action_77 _ = happyFail

action_78 (40) = happyShift action_87
action_78 (48) = happyShift action_53
action_78 (50) = happyShift action_54
action_78 (64) = happyShift action_88
action_78 (31) = happyGoto action_170
action_78 _ = happyFail

action_79 _ = happyReduce_18

action_80 _ = happyReduce_20

action_81 _ = happyReduce_22

action_82 _ = happyReduce_19

action_83 _ = happyReduce_21

action_84 _ = happyReduce_23

action_85 _ = happyReduce_9

action_86 (41) = happyShift action_108
action_86 (45) = happyShift action_109
action_86 (47) = happyShift action_110
action_86 (49) = happyShift action_111
action_86 (50) = happyShift action_112
action_86 (51) = happyShift action_113
action_86 (52) = happyShift action_114
action_86 (55) = happyShift action_115
action_86 (56) = happyShift action_116
action_86 (58) = happyShift action_117
action_86 (60) = happyShift action_118
action_86 (61) = happyShift action_119
action_86 _ = happyReduce_10

action_87 (40) = happyShift action_87
action_87 (48) = happyShift action_53
action_87 (50) = happyShift action_54
action_87 (64) = happyShift action_88
action_87 (31) = happyGoto action_169
action_87 _ = happyFail

action_88 (40) = happyShift action_168
action_88 _ = happyReduce_93

action_89 _ = happyReduce_35

action_90 (64) = happyShift action_167
action_90 _ = happyFail

action_91 _ = happyReduce_41

action_92 _ = happyReduce_25

action_93 (40) = happyShift action_87
action_93 (48) = happyShift action_53
action_93 (50) = happyShift action_54
action_93 (64) = happyShift action_88
action_93 (31) = happyGoto action_166
action_93 _ = happyFail

action_94 _ = happyReduce_27

action_95 _ = happyReduce_38

action_96 (64) = happyShift action_165
action_96 _ = happyFail

action_97 (42) = happyShift action_163
action_97 (64) = happyShift action_164
action_97 (23) = happyGoto action_162
action_97 _ = happyFail

action_98 (64) = happyShift action_58
action_98 (27) = happyGoto action_161
action_98 _ = happyFail

action_99 (61) = happyShift action_160
action_99 _ = happyFail

action_100 (36) = happyShift action_159
action_100 _ = happyFail

action_101 (39) = happyShift action_157
action_101 (40) = happyShift action_87
action_101 (48) = happyShift action_53
action_101 (50) = happyShift action_54
action_101 (64) = happyShift action_158
action_101 (28) = happyGoto action_154
action_101 (29) = happyGoto action_155
action_101 (30) = happyGoto action_156
action_101 (31) = happyGoto action_125
action_101 _ = happyFail

action_102 (40) = happyShift action_87
action_102 (44) = happyShift action_153
action_102 (48) = happyShift action_53
action_102 (50) = happyShift action_54
action_102 (64) = happyShift action_88
action_102 (30) = happyGoto action_152
action_102 (31) = happyGoto action_125
action_102 _ = happyFail

action_103 _ = happyReduce_89

action_104 (45) = happyShift action_109
action_104 (49) = happyShift action_111
action_104 (50) = happyShift action_112
action_104 (51) = happyShift action_113
action_104 (52) = happyShift action_114
action_104 (55) = happyShift action_115
action_104 (56) = happyShift action_116
action_104 (58) = happyShift action_117
action_104 (60) = happyShift action_118
action_104 (61) = happyShift action_119
action_104 _ = happyReduce_86

action_105 (44) = happyShift action_151
action_105 (46) = happyShift action_121
action_105 _ = happyFail

action_106 (41) = happyShift action_108
action_106 (44) = happyShift action_150
action_106 (45) = happyShift action_109
action_106 (47) = happyShift action_110
action_106 (49) = happyShift action_111
action_106 (50) = happyShift action_112
action_106 (51) = happyShift action_113
action_106 (52) = happyShift action_114
action_106 (55) = happyShift action_115
action_106 (56) = happyShift action_116
action_106 (58) = happyShift action_117
action_106 (60) = happyShift action_118
action_106 (61) = happyShift action_119
action_106 (62) = happyShift action_120
action_106 _ = happyFail

action_107 (64) = happyShift action_149
action_107 _ = happyFail

action_108 (40) = happyShift action_87
action_108 (48) = happyShift action_53
action_108 (50) = happyShift action_54
action_108 (64) = happyShift action_88
action_108 (31) = happyGoto action_148
action_108 _ = happyFail

action_109 (40) = happyShift action_87
action_109 (48) = happyShift action_53
action_109 (50) = happyShift action_54
action_109 (64) = happyShift action_88
action_109 (31) = happyGoto action_147
action_109 _ = happyFail

action_110 (40) = happyShift action_87
action_110 (48) = happyShift action_53
action_110 (50) = happyShift action_54
action_110 (64) = happyShift action_88
action_110 (31) = happyGoto action_146
action_110 _ = happyFail

action_111 (40) = happyShift action_87
action_111 (48) = happyShift action_53
action_111 (50) = happyShift action_54
action_111 (64) = happyShift action_88
action_111 (31) = happyGoto action_145
action_111 _ = happyFail

action_112 (40) = happyShift action_87
action_112 (48) = happyShift action_53
action_112 (50) = happyShift action_54
action_112 (64) = happyShift action_88
action_112 (31) = happyGoto action_144
action_112 _ = happyFail

action_113 (40) = happyShift action_87
action_113 (48) = happyShift action_53
action_113 (50) = happyShift action_54
action_113 (64) = happyShift action_88
action_113 (31) = happyGoto action_143
action_113 _ = happyFail

action_114 (40) = happyShift action_87
action_114 (48) = happyShift action_53
action_114 (50) = happyShift action_54
action_114 (64) = happyShift action_88
action_114 (31) = happyGoto action_142
action_114 _ = happyFail

action_115 (40) = happyShift action_87
action_115 (48) = happyShift action_53
action_115 (50) = happyShift action_54
action_115 (64) = happyShift action_88
action_115 (31) = happyGoto action_141
action_115 _ = happyFail

action_116 (40) = happyShift action_87
action_116 (48) = happyShift action_53
action_116 (50) = happyShift action_54
action_116 (64) = happyShift action_88
action_116 (31) = happyGoto action_140
action_116 _ = happyFail

action_117 (40) = happyShift action_87
action_117 (48) = happyShift action_53
action_117 (50) = happyShift action_54
action_117 (64) = happyShift action_88
action_117 (31) = happyGoto action_139
action_117 _ = happyFail

action_118 (40) = happyShift action_87
action_118 (48) = happyShift action_53
action_118 (50) = happyShift action_54
action_118 (64) = happyShift action_88
action_118 (31) = happyGoto action_138
action_118 _ = happyFail

action_119 (40) = happyShift action_87
action_119 (48) = happyShift action_53
action_119 (50) = happyShift action_54
action_119 (64) = happyShift action_88
action_119 (31) = happyGoto action_137
action_119 _ = happyFail

action_120 (33) = happyShift action_51
action_120 (40) = happyShift action_52
action_120 (48) = happyShift action_53
action_120 (50) = happyShift action_54
action_120 (64) = happyShift action_55
action_120 (21) = happyGoto action_136
action_120 (22) = happyGoto action_49
action_120 (31) = happyGoto action_50
action_120 _ = happyFail

action_121 (33) = happyShift action_51
action_121 (40) = happyShift action_52
action_121 (48) = happyShift action_53
action_121 (50) = happyShift action_54
action_121 (64) = happyShift action_55
action_121 (21) = happyGoto action_135
action_121 (22) = happyGoto action_49
action_121 (31) = happyGoto action_50
action_121 _ = happyFail

action_122 _ = happyReduce_43

action_123 _ = happyReduce_47

action_124 (35) = happyShift action_133
action_124 (39) = happyShift action_134
action_124 _ = happyFail

action_125 (41) = happyShift action_108
action_125 (45) = happyShift action_109
action_125 (47) = happyShift action_110
action_125 (49) = happyShift action_111
action_125 (50) = happyShift action_112
action_125 (51) = happyShift action_113
action_125 (52) = happyShift action_114
action_125 (55) = happyShift action_115
action_125 (56) = happyShift action_116
action_125 (58) = happyShift action_117
action_125 (60) = happyShift action_118
action_125 (61) = happyShift action_119
action_125 _ = happyReduce_76

action_126 _ = happyReduce_52

action_127 (53) = happyShift action_132
action_127 _ = happyFail

action_128 (53) = happyShift action_131
action_128 _ = happyFail

action_129 (53) = happyShift action_130
action_129 _ = happyFail

action_130 (40) = happyShift action_35
action_130 (64) = happyShift action_36
action_130 (65) = happyShift action_37
action_130 (66) = happyShift action_38
action_130 (69) = happyShift action_39
action_130 (18) = happyGoto action_199
action_130 _ = happyFail

action_131 (40) = happyShift action_35
action_131 (64) = happyShift action_36
action_131 (65) = happyShift action_37
action_131 (66) = happyShift action_38
action_131 (69) = happyShift action_39
action_131 (18) = happyGoto action_198
action_131 _ = happyFail

action_132 (40) = happyShift action_35
action_132 (64) = happyShift action_36
action_132 (65) = happyShift action_37
action_132 (66) = happyShift action_38
action_132 (69) = happyShift action_39
action_132 (18) = happyGoto action_197
action_132 _ = happyFail

action_133 (40) = happyShift action_87
action_133 (48) = happyShift action_53
action_133 (50) = happyShift action_54
action_133 (64) = happyShift action_88
action_133 (31) = happyGoto action_196
action_133 _ = happyFail

action_134 _ = happyReduce_53

action_135 _ = happyReduce_54

action_136 _ = happyReduce_59

action_137 (45) = happyShift action_109
action_137 (49) = happyFail
action_137 (50) = happyShift action_112
action_137 (51) = happyShift action_113
action_137 (52) = happyShift action_114
action_137 (55) = happyShift action_115
action_137 (56) = happyShift action_116
action_137 (58) = happyShift action_117
action_137 (60) = happyShift action_118
action_137 (61) = happyFail
action_137 _ = happyReduce_84

action_138 (45) = happyShift action_109
action_138 (50) = happyShift action_112
action_138 (51) = happyShift action_113
action_138 (52) = happyShift action_114
action_138 (55) = happyFail
action_138 (56) = happyFail
action_138 (58) = happyFail
action_138 (60) = happyFail
action_138 _ = happyReduce_83

action_139 (45) = happyShift action_109
action_139 (50) = happyShift action_112
action_139 (51) = happyShift action_113
action_139 (52) = happyShift action_114
action_139 (55) = happyFail
action_139 (56) = happyFail
action_139 (58) = happyFail
action_139 (60) = happyFail
action_139 _ = happyReduce_79

action_140 (45) = happyShift action_109
action_140 (50) = happyShift action_112
action_140 (51) = happyShift action_113
action_140 (52) = happyShift action_114
action_140 (55) = happyFail
action_140 (56) = happyFail
action_140 (58) = happyFail
action_140 (60) = happyFail
action_140 _ = happyReduce_80

action_141 (45) = happyShift action_109
action_141 (50) = happyShift action_112
action_141 (51) = happyShift action_113
action_141 (52) = happyShift action_114
action_141 (55) = happyFail
action_141 (56) = happyFail
action_141 (58) = happyFail
action_141 (60) = happyFail
action_141 _ = happyReduce_78

action_142 _ = happyReduce_90

action_143 _ = happyReduce_91

action_144 (51) = happyShift action_113
action_144 (52) = happyShift action_114
action_144 _ = happyReduce_88

action_145 (45) = happyShift action_109
action_145 (49) = happyFail
action_145 (50) = happyShift action_112
action_145 (51) = happyShift action_113
action_145 (52) = happyShift action_114
action_145 (55) = happyShift action_115
action_145 (56) = happyShift action_116
action_145 (58) = happyShift action_117
action_145 (60) = happyShift action_118
action_145 (61) = happyFail
action_145 _ = happyReduce_85

action_146 (45) = happyShift action_109
action_146 (49) = happyShift action_111
action_146 (50) = happyShift action_112
action_146 (51) = happyShift action_113
action_146 (52) = happyShift action_114
action_146 (55) = happyShift action_115
action_146 (56) = happyShift action_116
action_146 (58) = happyShift action_117
action_146 (60) = happyShift action_118
action_146 (61) = happyShift action_119
action_146 _ = happyReduce_81

action_147 (51) = happyShift action_113
action_147 (52) = happyShift action_114
action_147 _ = happyReduce_87

action_148 (45) = happyShift action_109
action_148 (47) = happyShift action_110
action_148 (49) = happyShift action_111
action_148 (50) = happyShift action_112
action_148 (51) = happyShift action_113
action_148 (52) = happyShift action_114
action_148 (55) = happyShift action_115
action_148 (56) = happyShift action_116
action_148 (58) = happyShift action_117
action_148 (60) = happyShift action_118
action_148 (61) = happyShift action_119
action_148 _ = happyReduce_82

action_149 (53) = happyShift action_195
action_149 _ = happyFail

action_150 _ = happyReduce_94

action_151 _ = happyReduce_61

action_152 (35) = happyShift action_133
action_152 (44) = happyShift action_194
action_152 _ = happyFail

action_153 _ = happyReduce_49

action_154 (35) = happyShift action_192
action_154 (39) = happyShift action_193
action_154 _ = happyFail

action_155 _ = happyReduce_73

action_156 (35) = happyShift action_133
action_156 (39) = happyShift action_191
action_156 _ = happyFail

action_157 _ = happyReduce_64

action_158 (40) = happyShift action_168
action_158 (54) = happyShift action_190
action_158 _ = happyReduce_93

action_159 (33) = happyShift action_51
action_159 (34) = happyShift action_189
action_159 (40) = happyShift action_52
action_159 (48) = happyShift action_53
action_159 (50) = happyShift action_54
action_159 (64) = happyShift action_55
action_159 (21) = happyGoto action_188
action_159 (22) = happyGoto action_49
action_159 (31) = happyGoto action_50
action_159 _ = happyFail

action_160 (33) = happyShift action_51
action_160 (40) = happyShift action_52
action_160 (48) = happyShift action_53
action_160 (50) = happyShift action_54
action_160 (64) = happyShift action_55
action_160 (21) = happyGoto action_187
action_160 (22) = happyGoto action_49
action_160 (31) = happyGoto action_50
action_160 _ = happyFail

action_161 _ = happyReduce_71

action_162 _ = happyReduce_72

action_163 (40) = happyShift action_87
action_163 (48) = happyShift action_53
action_163 (50) = happyShift action_54
action_163 (64) = happyShift action_88
action_163 (31) = happyGoto action_186
action_163 _ = happyFail

action_164 _ = happyReduce_65

action_165 (44) = happyShift action_185
action_165 _ = happyFail

action_166 (41) = happyShift action_108
action_166 (45) = happyShift action_109
action_166 (47) = happyShift action_110
action_166 (49) = happyShift action_111
action_166 (50) = happyShift action_112
action_166 (51) = happyShift action_113
action_166 (52) = happyShift action_114
action_166 (55) = happyShift action_115
action_166 (56) = happyShift action_116
action_166 (57) = happyShift action_184
action_166 (58) = happyShift action_117
action_166 (60) = happyShift action_118
action_166 (61) = happyShift action_119
action_166 _ = happyFail

action_167 (35) = happyShift action_183
action_167 _ = happyFail

action_168 (40) = happyShift action_87
action_168 (48) = happyShift action_53
action_168 (50) = happyShift action_54
action_168 (64) = happyShift action_88
action_168 (30) = happyGoto action_182
action_168 (31) = happyGoto action_125
action_168 _ = happyFail

action_169 (41) = happyShift action_108
action_169 (44) = happyShift action_150
action_169 (45) = happyShift action_109
action_169 (47) = happyShift action_110
action_169 (49) = happyShift action_111
action_169 (50) = happyShift action_112
action_169 (51) = happyShift action_113
action_169 (52) = happyShift action_114
action_169 (55) = happyShift action_115
action_169 (56) = happyShift action_116
action_169 (58) = happyShift action_117
action_169 (60) = happyShift action_118
action_169 (61) = happyShift action_119
action_169 _ = happyFail

action_170 (41) = happyShift action_108
action_170 (45) = happyShift action_109
action_170 (47) = happyShift action_110
action_170 (49) = happyShift action_111
action_170 (50) = happyShift action_112
action_170 (51) = happyShift action_113
action_170 (52) = happyShift action_114
action_170 (55) = happyShift action_115
action_170 (56) = happyShift action_116
action_170 (57) = happyShift action_181
action_170 (58) = happyShift action_117
action_170 (60) = happyShift action_118
action_170 (61) = happyShift action_119
action_170 _ = happyFail

action_171 (44) = happyShift action_180
action_171 _ = happyFail

action_172 (44) = happyShift action_179
action_172 _ = happyFail

action_173 (35) = happyShift action_178
action_173 _ = happyReduce_29

action_174 (63) = happyShift action_177
action_174 _ = happyFail

action_175 (52) = happyShift action_176
action_175 _ = happyReduce_32

action_176 (64) = happyShift action_175
action_176 (12) = happyGoto action_215
action_176 _ = happyFail

action_177 (64) = happyShift action_214
action_177 _ = happyFail

action_178 (64) = happyShift action_175
action_178 (10) = happyGoto action_213
action_178 (11) = happyGoto action_173
action_178 (12) = happyGoto action_174
action_178 _ = happyFail

action_179 _ = happyReduce_13

action_180 _ = happyReduce_11

action_181 (40) = happyShift action_87
action_181 (48) = happyShift action_53
action_181 (50) = happyShift action_54
action_181 (64) = happyShift action_88
action_181 (31) = happyGoto action_212
action_181 _ = happyFail

action_182 (35) = happyShift action_133
action_182 (44) = happyShift action_211
action_182 _ = happyFail

action_183 (64) = happyShift action_210
action_183 _ = happyFail

action_184 (40) = happyShift action_87
action_184 (48) = happyShift action_53
action_184 (50) = happyShift action_54
action_184 (64) = happyShift action_88
action_184 (31) = happyGoto action_209
action_184 _ = happyFail

action_185 _ = happyReduce_39

action_186 (41) = happyShift action_108
action_186 (45) = happyShift action_109
action_186 (47) = happyShift action_110
action_186 (49) = happyShift action_111
action_186 (50) = happyShift action_112
action_186 (51) = happyShift action_113
action_186 (52) = happyShift action_114
action_186 (55) = happyShift action_115
action_186 (56) = happyShift action_116
action_186 (57) = happyShift action_208
action_186 (58) = happyShift action_117
action_186 (60) = happyShift action_118
action_186 (61) = happyShift action_119
action_186 _ = happyFail

action_187 (46) = happyShift action_121
action_187 _ = happyReduce_15

action_188 _ = happyReduce_55

action_189 (40) = happyShift action_207
action_189 _ = happyFail

action_190 (40) = happyShift action_87
action_190 (48) = happyShift action_53
action_190 (50) = happyShift action_54
action_190 (64) = happyShift action_88
action_190 (31) = happyGoto action_206
action_190 _ = happyFail

action_191 _ = happyReduce_62

action_192 (64) = happyShift action_205
action_192 (29) = happyGoto action_204
action_192 _ = happyFail

action_193 _ = happyReduce_63

action_194 (41) = happyReduce_92
action_194 (44) = happyReduce_92
action_194 (45) = happyReduce_92
action_194 (47) = happyReduce_92
action_194 (49) = happyReduce_92
action_194 (50) = happyReduce_92
action_194 (51) = happyReduce_92
action_194 (52) = happyReduce_92
action_194 (55) = happyReduce_92
action_194 (56) = happyReduce_92
action_194 (58) = happyReduce_92
action_194 (60) = happyReduce_92
action_194 (61) = happyReduce_92
action_194 (62) = happyReduce_92
action_194 (63) = happyReduce_92
action_194 _ = happyReduce_50

action_195 (42) = happyShift action_163
action_195 (64) = happyShift action_164
action_195 (23) = happyGoto action_203
action_195 _ = happyFail

action_196 (41) = happyShift action_108
action_196 (45) = happyShift action_109
action_196 (47) = happyShift action_110
action_196 (49) = happyShift action_111
action_196 (50) = happyShift action_112
action_196 (51) = happyShift action_113
action_196 (52) = happyShift action_114
action_196 (55) = happyShift action_115
action_196 (56) = happyShift action_116
action_196 (58) = happyShift action_117
action_196 (60) = happyShift action_118
action_196 (61) = happyShift action_119
action_196 _ = happyReduce_77

action_197 (37) = happyShift action_47
action_197 (44) = happyShift action_202
action_197 _ = happyFail

action_198 (37) = happyShift action_47
action_198 (44) = happyShift action_201
action_198 _ = happyFail

action_199 (37) = happyShift action_47
action_199 (44) = happyShift action_200
action_199 _ = happyFail

action_200 _ = happyReduce_45

action_201 _ = happyReduce_46

action_202 _ = happyReduce_44

action_203 (35) = happyShift action_225
action_203 _ = happyFail

action_204 _ = happyReduce_74

action_205 (54) = happyShift action_190
action_205 _ = happyFail

action_206 (41) = happyShift action_108
action_206 (45) = happyShift action_109
action_206 (47) = happyShift action_110
action_206 (49) = happyShift action_111
action_206 (50) = happyShift action_112
action_206 (51) = happyShift action_113
action_206 (52) = happyShift action_114
action_206 (55) = happyShift action_115
action_206 (56) = happyShift action_116
action_206 (58) = happyShift action_117
action_206 (60) = happyShift action_118
action_206 (61) = happyShift action_119
action_206 _ = happyReduce_75

action_207 (40) = happyShift action_87
action_207 (48) = happyShift action_53
action_207 (50) = happyShift action_54
action_207 (64) = happyShift action_224
action_207 (24) = happyGoto action_220
action_207 (25) = happyGoto action_221
action_207 (31) = happyGoto action_222
action_207 (32) = happyGoto action_223
action_207 _ = happyFail

action_208 (40) = happyShift action_87
action_208 (48) = happyShift action_53
action_208 (50) = happyShift action_54
action_208 (64) = happyShift action_88
action_208 (31) = happyGoto action_219
action_208 _ = happyFail

action_209 (41) = happyShift action_108
action_209 (43) = happyShift action_218
action_209 (45) = happyShift action_109
action_209 (47) = happyShift action_110
action_209 (49) = happyShift action_111
action_209 (50) = happyShift action_112
action_209 (51) = happyShift action_113
action_209 (52) = happyShift action_114
action_209 (55) = happyShift action_115
action_209 (56) = happyShift action_116
action_209 (58) = happyShift action_117
action_209 (60) = happyShift action_118
action_209 (61) = happyShift action_119
action_209 _ = happyFail

action_210 (44) = happyShift action_217
action_210 _ = happyFail

action_211 _ = happyReduce_92

action_212 (41) = happyShift action_108
action_212 (43) = happyShift action_216
action_212 (45) = happyShift action_109
action_212 (47) = happyShift action_110
action_212 (49) = happyShift action_111
action_212 (50) = happyShift action_112
action_212 (51) = happyShift action_113
action_212 (52) = happyShift action_114
action_212 (55) = happyShift action_115
action_212 (56) = happyShift action_116
action_212 (58) = happyShift action_117
action_212 (60) = happyShift action_118
action_212 (61) = happyShift action_119
action_212 _ = happyFail

action_213 _ = happyReduce_30

action_214 _ = happyReduce_31

action_215 _ = happyReduce_33

action_216 _ = happyReduce_12

action_217 _ = happyReduce_36

action_218 _ = happyReduce_28

action_219 (41) = happyShift action_108
action_219 (43) = happyShift action_232
action_219 (45) = happyShift action_109
action_219 (47) = happyShift action_110
action_219 (49) = happyShift action_111
action_219 (50) = happyShift action_112
action_219 (51) = happyShift action_113
action_219 (52) = happyShift action_114
action_219 (55) = happyShift action_115
action_219 (56) = happyShift action_116
action_219 (58) = happyShift action_117
action_219 (60) = happyShift action_118
action_219 (61) = happyShift action_119
action_219 _ = happyFail

action_220 (53) = happyShift action_231
action_220 _ = happyFail

action_221 (37) = happyShift action_230
action_221 _ = happyReduce_67

action_222 (41) = happyShift action_108
action_222 (45) = happyShift action_109
action_222 (47) = happyShift action_110
action_222 (49) = happyShift action_111
action_222 (50) = happyShift action_112
action_222 (51) = happyShift action_113
action_222 (52) = happyShift action_114
action_222 (55) = happyShift action_115
action_222 (56) = happyShift action_116
action_222 (58) = happyShift action_117
action_222 (60) = happyShift action_118
action_222 (61) = happyShift action_119
action_222 (63) = happyShift action_229
action_222 _ = happyFail

action_223 (44) = happyShift action_228
action_223 _ = happyFail

action_224 (40) = happyShift action_168
action_224 (53) = happyShift action_227
action_224 _ = happyReduce_93

action_225 (33) = happyShift action_51
action_225 (40) = happyShift action_52
action_225 (48) = happyShift action_53
action_225 (50) = happyShift action_54
action_225 (64) = happyShift action_55
action_225 (21) = happyGoto action_226
action_225 (22) = happyGoto action_49
action_225 (31) = happyGoto action_50
action_225 _ = happyFail

action_226 (44) = happyShift action_238
action_226 (46) = happyShift action_121
action_226 _ = happyFail

action_227 (42) = happyShift action_163
action_227 (64) = happyShift action_164
action_227 (23) = happyGoto action_237
action_227 _ = happyFail

action_228 _ = happyReduce_57

action_229 (33) = happyShift action_51
action_229 (40) = happyShift action_52
action_229 (48) = happyShift action_53
action_229 (50) = happyShift action_54
action_229 (64) = happyShift action_55
action_229 (21) = happyGoto action_236
action_229 (22) = happyGoto action_49
action_229 (31) = happyGoto action_50
action_229 _ = happyFail

action_230 (64) = happyShift action_235
action_230 (24) = happyGoto action_234
action_230 (25) = happyGoto action_221
action_230 _ = happyFail

action_231 (33) = happyShift action_51
action_231 (40) = happyShift action_52
action_231 (48) = happyShift action_53
action_231 (50) = happyShift action_54
action_231 (64) = happyShift action_55
action_231 (21) = happyGoto action_233
action_231 (22) = happyGoto action_49
action_231 (31) = happyGoto action_50
action_231 _ = happyFail

action_232 _ = happyReduce_66

action_233 (44) = happyShift action_241
action_233 (46) = happyShift action_121
action_233 _ = happyFail

action_234 _ = happyReduce_68

action_235 (53) = happyShift action_227
action_235 _ = happyFail

action_236 (46) = happyShift action_240
action_236 _ = happyReduce_95

action_237 (35) = happyShift action_239
action_237 _ = happyFail

action_238 _ = happyReduce_58

action_239 (40) = happyShift action_87
action_239 (48) = happyShift action_53
action_239 (50) = happyShift action_54
action_239 (64) = happyShift action_88
action_239 (31) = happyGoto action_244
action_239 _ = happyFail

action_240 (33) = happyShift action_51
action_240 (40) = happyShift action_52
action_240 (48) = happyShift action_53
action_240 (50) = happyShift action_54
action_240 (64) = happyShift action_55
action_240 (21) = happyGoto action_135
action_240 (22) = happyGoto action_49
action_240 (31) = happyGoto action_242
action_240 (32) = happyGoto action_243
action_240 _ = happyFail

action_241 _ = happyReduce_56

action_242 (41) = happyShift action_108
action_242 (45) = happyShift action_109
action_242 (47) = happyShift action_110
action_242 (49) = happyShift action_111
action_242 (50) = happyShift action_112
action_242 (51) = happyShift action_113
action_242 (52) = happyShift action_114
action_242 (55) = happyShift action_115
action_242 (56) = happyShift action_116
action_242 (58) = happyShift action_117
action_242 (60) = happyShift action_118
action_242 (61) = happyShift action_119
action_242 (62) = happyShift action_120
action_242 (63) = happyShift action_229
action_242 _ = happyFail

action_243 _ = happyReduce_96

action_244 (41) = happyShift action_108
action_244 (45) = happyShift action_109
action_244 (47) = happyShift action_110
action_244 (49) = happyShift action_111
action_244 (50) = happyShift action_112
action_244 (51) = happyShift action_113
action_244 (52) = happyShift action_114
action_244 (55) = happyShift action_115
action_244 (56) = happyShift action_116
action_244 (58) = happyShift action_117
action_244 (60) = happyShift action_118
action_244 (61) = happyShift action_119
action_244 _ = happyReduce_69

happyReduce_1 = happySpecReduce_1  4 happyReduction_1
happyReduction_1 (HappyAbsSyn5  happy_var_1)
	 =  HappyAbsSyn4
		 (happy_var_1
	)
happyReduction_1 _  = notHappyAtAll 

happyReduce_2 = happySpecReduce_0  5 happyReduction_2
happyReduction_2  =  HappyAbsSyn5
		 ([]
	)

happyReduce_3 = happySpecReduce_2  5 happyReduction_3
happyReduction_3 (HappyAbsSyn5  happy_var_2)
	(HappyAbsSyn6  happy_var_1)
	 =  HappyAbsSyn5
		 (happy_var_1 : happy_var_2
	)
happyReduction_3 _ _  = notHappyAtAll 

happyReduce_4 = happySpecReduce_2  6 happyReduction_4
happyReduction_4 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataHiding happy_var_2
	)
happyReduction_4 _ _  = notHappyAtAll 

happyReduce_5 = happySpecReduce_2  6 happyReduction_5
happyReduction_5 (HappyAbsSyn15  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataRenaming happy_var_2
	)
happyReduction_5 _ _  = notHappyAtAll 

happyReduce_6 = happySpecReduce_2  6 happyReduction_6
happyReduction_6 (HappyAbsSyn8  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataActions happy_var_2
	)
happyReduction_6 _ _  = notHappyAtAll 

happyReduce_7 = happySpecReduce_2  6 happyReduction_7
happyReduction_7 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataEncapsulation happy_var_2
	)
happyReduction_7 _ _  = notHappyAtAll 

happyReduce_8 = happySpecReduce_2  6 happyReduction_8
happyReduction_8 (HappyAbsSyn13  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataCommunication happy_var_2
	)
happyReduction_8 _ _  = notHappyAtAll 

happyReduce_9 = happyReduce 4 6 happyReduction_9
happyReduction_9 (_ `HappyStk`
	(HappyAbsSyn7  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataUntilFormula happy_var_3
	) `HappyStk` happyRest

happyReduce_10 = happyReduce 4 6 happyReduction_10
happyReduction_10 ((HappyAbsSyn31  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataConstant happy_var_2 happy_var_4
	) `HappyStk` happyRest

happyReduce_11 = happyReduce 6 6 happyReduction_11
happyReduction_11 (_ `HappyStk`
	(HappyAbsSyn17  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataEnumType happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_12 = happyReduce 8 6 happyReduction_12
happyReduction_12 (_ `HappyStk`
	(HappyAbsSyn31  happy_var_7) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn31  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataRangeType happy_var_2 happy_var_5 happy_var_7
	) `HappyStk` happyRest

happyReduce_13 = happyReduce 6 6 happyReduction_13
happyReduction_13 (_ `HappyStk`
	(HappyAbsSyn10  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataFunction happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_14 = happySpecReduce_2  6 happyReduction_14
happyReduction_14 (HappyAbsSyn18  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (ParserInitialProcess happy_var_2
	)
happyReduction_14 _ _  = notHappyAtAll 

happyReduce_15 = happyReduce 6 6 happyReduction_15
happyReduction_15 ((HappyAbsSyn21  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn26  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (ParserProcess (Process happy_var_1 (reverse happy_var_3) happy_var_6)
	) `HappyStk` happyRest

happyReduce_16 = happySpecReduce_3  6 happyReduction_16
happyReduction_16 (HappyAbsSyn21  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn6
		 (ParserProcess (Process happy_var_1 [] happy_var_3)
	)
happyReduction_16 _ _ _  = notHappyAtAll 

happyReduce_17 = happySpecReduce_0  7 happyReduction_17
happyReduction_17  =  HappyAbsSyn7
		 (""
	)

happyReduce_18 = happySpecReduce_2  7 happyReduction_18
happyReduction_18 (HappyAbsSyn7  happy_var_2)
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn7
		 (happy_var_1 ++ " " ++ happy_var_2
	)
happyReduction_18 _ _  = notHappyAtAll 

happyReduce_19 = happySpecReduce_2  7 happyReduction_19
happyReduction_19 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("|" ++ " " ++ happy_var_2
	)
happyReduction_19 _ _  = notHappyAtAll 

happyReduce_20 = happySpecReduce_2  7 happyReduction_20
happyReduction_20 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("!" ++ " " ++ happy_var_2
	)
happyReduction_20 _ _  = notHappyAtAll 

happyReduce_21 = happySpecReduce_2  7 happyReduction_21
happyReduction_21 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("(" ++ " " ++ happy_var_2
	)
happyReduction_21 _ _  = notHappyAtAll 

happyReduce_22 = happySpecReduce_2  7 happyReduction_22
happyReduction_22 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 (")" ++ " " ++ happy_var_2
	)
happyReduction_22 _ _  = notHappyAtAll 

happyReduce_23 = happySpecReduce_2  7 happyReduction_23
happyReduction_23 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("," ++ " " ++ happy_var_2
	)
happyReduction_23 _ _  = notHappyAtAll 

happyReduce_24 = happySpecReduce_1  8 happyReduction_24
happyReduction_24 (HappyAbsSyn9  happy_var_1)
	 =  HappyAbsSyn8
		 ([happy_var_1]
	)
happyReduction_24 _  = notHappyAtAll 

happyReduce_25 = happySpecReduce_3  8 happyReduction_25
happyReduction_25 (HappyAbsSyn8  happy_var_3)
	_
	(HappyAbsSyn9  happy_var_1)
	 =  HappyAbsSyn8
		 (happy_var_1:happy_var_3
	)
happyReduction_25 _ _ _  = notHappyAtAll 

happyReduce_26 = happySpecReduce_1  9 happyReduction_26
happyReduction_26 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn9
		 ((happy_var_1, None)
	)
happyReduction_26 _  = notHappyAtAll 

happyReduce_27 = happySpecReduce_3  9 happyReduction_27
happyReduction_27 _
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn9
		 ((happy_var_1, Bool)
	)
happyReduction_27 _ _ _  = notHappyAtAll 

happyReduce_28 = happyReduce 7 9 happyReduction_28
happyReduction_28 (_ `HappyStk`
	(HappyAbsSyn31  happy_var_6) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn31  happy_var_4) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn9
		 ((happy_var_1, IntRange happy_var_4 happy_var_6)
	) `HappyStk` happyRest

happyReduce_29 = happySpecReduce_1  10 happyReduction_29
happyReduction_29 (HappyAbsSyn11  happy_var_1)
	 =  HappyAbsSyn10
		 ([happy_var_1]
	)
happyReduction_29 _  = notHappyAtAll 

happyReduce_30 = happySpecReduce_3  10 happyReduction_30
happyReduction_30 (HappyAbsSyn10  happy_var_3)
	_
	(HappyAbsSyn11  happy_var_1)
	 =  HappyAbsSyn10
		 (happy_var_1:happy_var_3
	)
happyReduction_30 _ _ _  = notHappyAtAll 

happyReduce_31 = happySpecReduce_3  11 happyReduction_31
happyReduction_31 (HappyTerminal (TokenString happy_var_3))
	_
	(HappyAbsSyn12  happy_var_1)
	 =  HappyAbsSyn11
		 ((happy_var_1, happy_var_3)
	)
happyReduction_31 _ _ _  = notHappyAtAll 

happyReduce_32 = happySpecReduce_1  12 happyReduction_32
happyReduction_32 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn12
		 ([happy_var_1]
	)
happyReduction_32 _  = notHappyAtAll 

happyReduce_33 = happySpecReduce_3  12 happyReduction_33
happyReduction_33 (HappyAbsSyn12  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn12
		 (happy_var_1:happy_var_3
	)
happyReduction_33 _ _ _  = notHappyAtAll 

happyReduce_34 = happySpecReduce_1  13 happyReduction_34
happyReduction_34 (HappyAbsSyn14  happy_var_1)
	 =  HappyAbsSyn13
		 ([happy_var_1]
	)
happyReduction_34 _  = notHappyAtAll 

happyReduce_35 = happySpecReduce_3  13 happyReduction_35
happyReduction_35 (HappyAbsSyn13  happy_var_3)
	_
	(HappyAbsSyn14  happy_var_1)
	 =  HappyAbsSyn13
		 (happy_var_1 : happy_var_3
	)
happyReduction_35 _ _ _  = notHappyAtAll 

happyReduce_36 = happyReduce 7 14 happyReduction_36
happyReduction_36 (_ `HappyStk`
	(HappyTerminal (TokenString happy_var_6)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_4)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn14
		 ((happy_var_2,happy_var_4,happy_var_6)
	) `HappyStk` happyRest

happyReduce_37 = happySpecReduce_1  15 happyReduction_37
happyReduction_37 (HappyAbsSyn16  happy_var_1)
	 =  HappyAbsSyn15
		 ([happy_var_1]
	)
happyReduction_37 _  = notHappyAtAll 

happyReduce_38 = happySpecReduce_3  15 happyReduction_38
happyReduction_38 (HappyAbsSyn15  happy_var_3)
	_
	(HappyAbsSyn16  happy_var_1)
	 =  HappyAbsSyn15
		 (happy_var_1 : happy_var_3
	)
happyReduction_38 _ _ _  = notHappyAtAll 

happyReduce_39 = happyReduce 5 16 happyReduction_39
happyReduction_39 (_ `HappyStk`
	(HappyTerminal (TokenString happy_var_4)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn16
		 ((happy_var_2,happy_var_4)
	) `HappyStk` happyRest

happyReduce_40 = happySpecReduce_1  17 happyReduction_40
happyReduction_40 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn17
		 ([happy_var_1]
	)
happyReduction_40 _  = notHappyAtAll 

happyReduce_41 = happySpecReduce_3  17 happyReduction_41
happyReduction_41 (HappyAbsSyn17  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn17
		 (happy_var_1 : happy_var_3
	)
happyReduction_41 _ _ _  = notHappyAtAll 

happyReduce_42 = happySpecReduce_2  18 happyReduction_42
happyReduction_42 (HappyAbsSyn20  happy_var_2)
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn18
		 (InitSingleProcess (InitialProcess happy_var_1 happy_var_2)
	)
happyReduction_42 _ _  = notHappyAtAll 

happyReduce_43 = happySpecReduce_3  18 happyReduction_43
happyReduction_43 (HappyAbsSyn18  happy_var_3)
	_
	(HappyAbsSyn18  happy_var_1)
	 =  HappyAbsSyn18
		 (InitParallel happy_var_1 happy_var_3
	)
happyReduction_43 _ _ _  = notHappyAtAll 

happyReduce_44 = happyReduce 6 18 happyReduction_44
happyReduction_44 (_ `HappyStk`
	(HappyAbsSyn18  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitHiding happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_45 = happyReduce 6 18 happyReduction_45
happyReduction_45 (_ `HappyStk`
	(HappyAbsSyn18  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitEncapsulation happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_46 = happyReduce 6 18 happyReduction_46
happyReduction_46 (_ `HappyStk`
	(HappyAbsSyn18  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn15  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitRenaming happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_47 = happySpecReduce_3  18 happyReduction_47
happyReduction_47 _
	(HappyAbsSyn18  happy_var_2)
	_
	 =  HappyAbsSyn18
		 (happy_var_2
	)
happyReduction_47 _ _ _  = notHappyAtAll 

happyReduce_48 = happySpecReduce_0  19 happyReduction_48
happyReduction_48  =  HappyAbsSyn19
		 ([]
	)

happyReduce_49 = happySpecReduce_2  19 happyReduction_49
happyReduction_49 _
	_
	 =  HappyAbsSyn19
		 ([]
	)

happyReduce_50 = happySpecReduce_3  19 happyReduction_50
happyReduction_50 _
	(HappyAbsSyn30  happy_var_2)
	_
	 =  HappyAbsSyn19
		 (reverse happy_var_2
	)
happyReduction_50 _ _ _  = notHappyAtAll 

happyReduce_51 = happySpecReduce_0  20 happyReduction_51
happyReduction_51  =  HappyAbsSyn20
		 ([]
	)

happyReduce_52 = happySpecReduce_2  20 happyReduction_52
happyReduction_52 _
	_
	 =  HappyAbsSyn20
		 ([]
	)

happyReduce_53 = happySpecReduce_3  20 happyReduction_53
happyReduction_53 _
	(HappyAbsSyn30  happy_var_2)
	_
	 =  HappyAbsSyn20
		 (reverse happy_var_2
	)
happyReduction_53 _ _ _  = notHappyAtAll 

happyReduce_54 = happySpecReduce_3  21 happyReduction_54
happyReduction_54 (HappyAbsSyn21  happy_var_3)
	_
	(HappyAbsSyn21  happy_var_1)
	 =  HappyAbsSyn21
		 (Plus ([happy_var_1] ++ [happy_var_3])
	)
happyReduction_54 _ _ _  = notHappyAtAll 

happyReduce_55 = happyReduce 4 21 happyReduction_55
happyReduction_55 ((HappyAbsSyn21  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix happy_var_1 happy_var_2 [("i0", TypeRange 1 1, (Variable "1"))] happy_var_4
	) `HappyStk` happyRest

happyReduce_56 = happyReduce 9 21 happyReduction_56
happyReduction_56 (_ `HappyStk`
	(HappyAbsSyn21  happy_var_8) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn24  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix happy_var_1 happy_var_2 happy_var_6 happy_var_8
	) `HappyStk` happyRest

happyReduce_57 = happyReduce 7 21 happyReduction_57
happyReduction_57 (_ `HappyStk`
	(HappyAbsSyn32  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix2 happy_var_1 happy_var_2 happy_var_6
	) `HappyStk` happyRest

happyReduce_58 = happyReduce 8 21 happyReduction_58
happyReduction_58 (_ `HappyStk`
	(HappyAbsSyn21  happy_var_7) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn23  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_3)) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (Sum happy_var_3 happy_var_5 happy_var_7
	) `HappyStk` happyRest

happyReduce_59 = happySpecReduce_3  21 happyReduction_59
happyReduction_59 (HappyAbsSyn21  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn21
		 (Implication happy_var_1 happy_var_3
	)
happyReduction_59 _ _ _  = notHappyAtAll 

happyReduce_60 = happySpecReduce_1  21 happyReduction_60
happyReduction_60 (HappyAbsSyn22  happy_var_1)
	 =  HappyAbsSyn21
		 (happy_var_1
	)
happyReduction_60 _  = notHappyAtAll 

happyReduce_61 = happySpecReduce_3  21 happyReduction_61
happyReduction_61 _
	(HappyAbsSyn21  happy_var_2)
	_
	 =  HappyAbsSyn21
		 (happy_var_2
	)
happyReduction_61 _ _ _  = notHappyAtAll 

happyReduce_62 = happyReduce 4 22 happyReduction_62
happyReduction_62 (_ `HappyStk`
	(HappyAbsSyn30  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn22
		 (ProcessInstance happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_63 = happyReduce 4 22 happyReduction_63
happyReduction_63 (_ `HappyStk`
	(HappyAbsSyn28  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn22
		 (ProcessInstance2 happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_64 = happySpecReduce_3  22 happyReduction_64
happyReduction_64 _
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn22
		 (ProcessInstance2 happy_var_1 []
	)
happyReduction_64 _ _ _  = notHappyAtAll 

happyReduce_65 = happySpecReduce_1  23 happyReduction_65
happyReduction_65 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn23
		 (TypeName happy_var_1
	)
happyReduction_65 _  = notHappyAtAll 

happyReduce_66 = happyReduce 5 23 happyReduction_66
happyReduction_66 (_ `HappyStk`
	(HappyAbsSyn31  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn31  happy_var_2) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (TypeRangeExpressions happy_var_2 happy_var_4
	) `HappyStk` happyRest

happyReduce_67 = happySpecReduce_1  24 happyReduction_67
happyReduction_67 (HappyAbsSyn25  happy_var_1)
	 =  HappyAbsSyn24
		 ([happy_var_1]
	)
happyReduction_67 _  = notHappyAtAll 

happyReduce_68 = happySpecReduce_3  24 happyReduction_68
happyReduction_68 (HappyAbsSyn24  happy_var_3)
	_
	(HappyAbsSyn25  happy_var_1)
	 =  HappyAbsSyn24
		 (happy_var_1 : happy_var_3
	)
happyReduction_68 _ _ _  = notHappyAtAll 

happyReduce_69 = happyReduce 5 25 happyReduction_69
happyReduction_69 ((HappyAbsSyn31  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn23  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn25
		 ((happy_var_1, happy_var_3, happy_var_5)
	) `HappyStk` happyRest

happyReduce_70 = happySpecReduce_1  26 happyReduction_70
happyReduction_70 (HappyAbsSyn27  happy_var_1)
	 =  HappyAbsSyn26
		 ([happy_var_1]
	)
happyReduction_70 _  = notHappyAtAll 

happyReduce_71 = happySpecReduce_3  26 happyReduction_71
happyReduction_71 (HappyAbsSyn27  happy_var_3)
	_
	(HappyAbsSyn26  happy_var_1)
	 =  HappyAbsSyn26
		 (happy_var_3 : happy_var_1
	)
happyReduction_71 _ _ _  = notHappyAtAll 

happyReduce_72 = happySpecReduce_3  27 happyReduction_72
happyReduction_72 (HappyAbsSyn23  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn27
		 ((happy_var_1, happy_var_3)
	)
happyReduction_72 _ _ _  = notHappyAtAll 

happyReduce_73 = happySpecReduce_1  28 happyReduction_73
happyReduction_73 (HappyAbsSyn29  happy_var_1)
	 =  HappyAbsSyn28
		 ([happy_var_1]
	)
happyReduction_73 _  = notHappyAtAll 

happyReduce_74 = happySpecReduce_3  28 happyReduction_74
happyReduction_74 (HappyAbsSyn29  happy_var_3)
	_
	(HappyAbsSyn28  happy_var_1)
	 =  HappyAbsSyn28
		 (happy_var_3 : happy_var_1
	)
happyReduction_74 _ _ _  = notHappyAtAll 

happyReduce_75 = happySpecReduce_3  29 happyReduction_75
happyReduction_75 (HappyAbsSyn31  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn29
		 ((happy_var_1, happy_var_3)
	)
happyReduction_75 _ _ _  = notHappyAtAll 

happyReduce_76 = happySpecReduce_1  30 happyReduction_76
happyReduction_76 (HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn30
		 ([happy_var_1]
	)
happyReduction_76 _  = notHappyAtAll 

happyReduce_77 = happySpecReduce_3  30 happyReduction_77
happyReduction_77 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn30  happy_var_1)
	 =  HappyAbsSyn30
		 (happy_var_3 : happy_var_1
	)
happyReduction_77 _ _ _  = notHappyAtAll 

happyReduce_78 = happySpecReduce_3  31 happyReduction_78
happyReduction_78 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "lt" [happy_var_1, happy_var_3]
	)
happyReduction_78 _ _ _  = notHappyAtAll 

happyReduce_79 = happySpecReduce_3  31 happyReduction_79
happyReduction_79 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "leq" [happy_var_1, happy_var_3]
	)
happyReduction_79 _ _ _  = notHappyAtAll 

happyReduce_80 = happySpecReduce_3  31 happyReduction_80
happyReduction_80 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "gt" [happy_var_1, happy_var_3]
	)
happyReduction_80 _ _ _  = notHappyAtAll 

happyReduce_81 = happySpecReduce_3  31 happyReduction_81
happyReduction_81 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "and" [happy_var_1, happy_var_3]
	)
happyReduction_81 _ _ _  = notHappyAtAll 

happyReduce_82 = happySpecReduce_3  31 happyReduction_82
happyReduction_82 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "or" [happy_var_1, happy_var_3]
	)
happyReduction_82 _ _ _  = notHappyAtAll 

happyReduce_83 = happySpecReduce_3  31 happyReduction_83
happyReduction_83 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "geq" [happy_var_1, happy_var_3]
	)
happyReduction_83 _ _ _  = notHappyAtAll 

happyReduce_84 = happySpecReduce_3  31 happyReduction_84
happyReduction_84 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "eq" [happy_var_1, happy_var_3]
	)
happyReduction_84 _ _ _  = notHappyAtAll 

happyReduce_85 = happySpecReduce_3  31 happyReduction_85
happyReduction_85 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "not" [Function "eq" [happy_var_1, happy_var_3]]
	)
happyReduction_85 _ _ _  = notHappyAtAll 

happyReduce_86 = happySpecReduce_2  31 happyReduction_86
happyReduction_86 (HappyAbsSyn31  happy_var_2)
	_
	 =  HappyAbsSyn31
		 (Function "not" [happy_var_2]
	)
happyReduction_86 _ _  = notHappyAtAll 

happyReduce_87 = happySpecReduce_3  31 happyReduction_87
happyReduction_87 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "plus" [happy_var_1, happy_var_3]
	)
happyReduction_87 _ _ _  = notHappyAtAll 

happyReduce_88 = happySpecReduce_3  31 happyReduction_88
happyReduction_88 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "minus" [happy_var_1, happy_var_3]
	)
happyReduction_88 _ _ _  = notHappyAtAll 

happyReduce_89 = happySpecReduce_2  31 happyReduction_89
happyReduction_89 (HappyAbsSyn31  happy_var_2)
	_
	 =  HappyAbsSyn31
		 (Function "multiply" [Variable "-1", happy_var_2]
	)
happyReduction_89 _ _  = notHappyAtAll 

happyReduce_90 = happySpecReduce_3  31 happyReduction_90
happyReduction_90 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "multiply" [happy_var_1, happy_var_3]
	)
happyReduction_90 _ _ _  = notHappyAtAll 

happyReduce_91 = happySpecReduce_3  31 happyReduction_91
happyReduction_91 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn31
		 (Function "divide" [happy_var_1, happy_var_3]
	)
happyReduction_91 _ _ _  = notHappyAtAll 

happyReduce_92 = happyReduce 4 31 happyReduction_92
happyReduction_92 (_ `HappyStk`
	(HappyAbsSyn30  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn31
		 (Function happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_93 = happySpecReduce_1  31 happyReduction_93
happyReduction_93 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn31
		 (Variable happy_var_1
	)
happyReduction_93 _  = notHappyAtAll 

happyReduce_94 = happySpecReduce_3  31 happyReduction_94
happyReduction_94 _
	(HappyAbsSyn31  happy_var_2)
	_
	 =  HappyAbsSyn31
		 (happy_var_2
	)
happyReduction_94 _ _ _  = notHappyAtAll 

happyReduce_95 = happySpecReduce_3  32 happyReduction_95
happyReduction_95 (HappyAbsSyn21  happy_var_3)
	_
	(HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn32
		 ([(happy_var_1, happy_var_3)]
	)
happyReduction_95 _ _ _  = notHappyAtAll 

happyReduce_96 = happyReduce 5 32 happyReduction_96
happyReduction_96 ((HappyAbsSyn32  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn31  happy_var_1) `HappyStk`
	happyRest)
	 = HappyAbsSyn32
		 ((happy_var_1, happy_var_3) : happy_var_5
	) `HappyStk` happyRest

happyNewToken action sts stk
	= lexer(\tk -> 
	let cont i = action i i tk (HappyState action) sts stk in
	case tk of {
	TokenEOF -> action 75 75 tk (HappyState action) sts stk;
	TokenSum -> cont 33;
	TokenPSum -> cont 34;
	TokenComma -> cont 35;
	TokenSeq -> cont 36;
	TokenIndependence -> cont 37;
	TokenOSB -> cont 38;
	TokenCSB -> cont 39;
	TokenOB -> cont 40;
	TokenOr -> cont 41;
	TokenOA -> cont 42;
	TokenCA -> cont 43;
	TokenCB -> cont 44;
	TokenPlus -> cont 45;
	TokenPlusPlus -> cont 46;
	TokenAnd -> cont 47;
	TokenNot -> cont 48;
	TokenNotEqual -> cont 49;
	TokenMinus -> cont 50;
	TokenDivide -> cont 51;
	TokenMultiply -> cont 52;
	TokenColon -> cont 53;
	TokenBecomes -> cont 54;
	TokenSmaller -> cont 55;
	TokenGreater -> cont 56;
	TokenDotDot -> cont 57;
	TokenSmallerEq -> cont 58;
	TokenInit -> cont 59;
	TokenGreaterEq -> cont 60;
	TokenEqual -> cont 61;
	TokenImplies -> cont 62;
	TokenProbDef -> cont 63;
	TokenString happy_dollar_dollar -> cont 64;
	TokenHide -> cont 65;
	TokenRename -> cont 66;
	TokenBool -> cont 67;
	TokenActions -> cont 68;
	TokenEncap -> cont 69;
	TokenComm -> cont 70;
	TokenConstant -> cont 71;
	TokenUntilFormula -> cont 72;
	TokenType -> cont 73;
	TokenFunction -> cont 74;
	_ -> happyError' tk
	})

happyError_ tk = happyError' tk

happyThen :: () => ParserMonad a -> (a -> ParserMonad b) -> ParserMonad b
happyThen = (thenParserMonad)
happyReturn :: () => a -> ParserMonad a
happyReturn = (returnParserMonad)
happyThen1 = happyThen
happyReturn1 :: () => a -> ParserMonad a
happyReturn1 = happyReturn
happyError' :: () => (Token) -> ParserMonad a
happyError' tk = parseError tk

parse = happySomeParser where
  happySomeParser = happyThen (happyParse action_0) (\x -> case x of {HappyAbsSyn4 z -> happyReturn z; _other -> notHappyAtAll })

happySeq = happyDontSeq


parseError :: Token -> ParserMonad a 
parseError t = failParserMonad ("Error: Parser did not expect token \"" ++ show t ++ "\"")


data Token = TokenSum
           | TokenSeq  
           | TokenString String
           | TokenIndependence
           | TokenUntilFormula
           | TokenOB 
           | TokenCB 
           | TokenOSB 
           | TokenCSB 
           | TokenOA
           | TokenCA
           | TokenOr
           | TokenImplies
           | TokenColon
           | TokenBool
           | TokenPlus
           | TokenPlusPlus
           | TokenMinus
           | TokenDivide
           | TokenMultiply
           | TokenThen
           | TokenElse
           | TokenActions
           | TokenComma
           | TokenEqual
           | TokenNotEqual
           | TokenNot
           | TokenPSum
           | TokenDotDot
           | TokenHide
           | TokenRename
           | TokenEncap
           | TokenAnd
           | TokenComm
           | TokenConstant
           | TokenType
           | TokenFunction
           | TokenInit
           | TokenSmaller
           | TokenGreater
           | TokenSmallerEq
           | TokenGreaterEq
           | TokenProbDef
           | TokenBecomes
           | TokenEOF
instance Show Token where
  show x =
   case x of 
     TokenSum -> "sum("
     TokenSeq -> "."
     TokenString s -> s
     TokenIndependence -> "||"
     TokenUntilFormula -> "U"
     TokenOB  -> "("
     TokenCB  -> ")"
     TokenOSB -> "["
     TokenCSB -> "]"
     TokenOA  -> "{"
     TokenCA  -> "}"
     TokenOr -> "|"
     TokenImplies -> "=>"
     TokenColon -> ":"
     TokenBool -> "Bool"
     TokenPlus -> "+"
     TokenPlusPlus -> "++"
     TokenMinus -> "-"
     TokenDivide -> "/"
     TokenMultiply -> "*"
     TokenThen -> "<|"
     TokenElse -> "|>"
     TokenActions -> "actions"
     TokenComma -> ","
     TokenEqual -> "="
     TokenNotEqual -> "!="
     TokenNot -> "!"
     TokenPSum -> "psum"
     TokenDotDot -> ".."
     TokenHide -> "hide"
     TokenEncap -> "encap"
     TokenAnd -> "&"
     TokenComm -> "comm"
     TokenConstant -> "constant"
     TokenType -> "type"
     TokenFunction -> "function"
     TokenInit -> "init"
     TokenSmaller -> "<"
     TokenGreater -> ">"
     TokenSmallerEq -> "<="
     TokenGreaterEq -> ">="
     TokenProbDef -> "->"
     TokenBecomes -> ":="
     TokenEOF -> "EOF"

-- lexer :: String -> [Token] 
lexer :: (Token -> ParserMonad a) -> ParserMonad a
lexer cont ('\n':cs) = \line -> lexer cont cs (line + 1)
lexer cont ('\r':cs) = \line -> lexer cont cs line
lexer cont [] = cont TokenEOF [] 
lexer cont ('.':'.':cs) = cont TokenDotDot cs
lexer cont ('i':'n':'i':'t':' ':cs) = cont TokenInit cs
lexer cont ('i':'n':'i':'t':'\n':cs) = cont TokenInit ('\n':cs)
lexer cont ('i':'n':'i':'t':'\r':cs) = cont TokenInit ('\r':cs)
lexer cont ('i':'n':'i':'t':'\t':cs) = cont TokenInit ('\t':cs)
lexer cont ('u':'n':'t':'i':'l':' ':cs) = cont TokenUntilFormula cs
lexer cont ('u':'n':'t':'i':'l':'\n':cs) = cont TokenUntilFormula ('\n':cs)
lexer cont ('u':'n':'t':'i':'l':'\r':cs) = cont TokenUntilFormula ('\r':cs)
lexer cont ('u':'n':'t':'i':'l':'\t':cs) = cont TokenUntilFormula ('\t':cs)
lexer cont ('b':'o':'o':'l':' ':cs) = cont TokenBool cs
lexer cont ('b':'o':'o':'l':'\n':cs) = cont TokenBool ('\n':cs)
lexer cont ('b':'o':'o':'l':'\r':cs) = cont TokenBool ('\r':cs)
lexer cont ('b':'o':'o':'l':'\t':cs) = cont TokenBool ('\t':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':' ':cs) = cont TokenActions cs
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\n':cs) = cont TokenActions ('\n':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\r':cs) = cont TokenActions ('\r':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\t':cs) = cont TokenActions ('\t':cs)
lexer cont ('h':'i':'d':'e':' ':cs) = cont TokenHide cs
lexer cont ('h':'i':'d':'e':'\n':cs) = cont TokenHide ('\n':cs)
lexer cont ('h':'i':'d':'e':'\r':cs) = cont TokenHide ('\r':cs)
lexer cont ('h':'i':'d':'e':'\t':cs) = cont TokenHide ('\t':cs)
lexer cont ('h':'i':'d':'e':'(':cs) = cont TokenHide ('(':cs)
lexer cont ('s':'u':'m':cs) | not (isAlpha (head cs)) = cont TokenSum cs
lexer cont ('p':'s':'u':'m':cs) | not (isAlpha (head cs)) = cont TokenPSum cs
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':' ':cs) = cont TokenConstant cs
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\n':cs) = cont TokenConstant ('\n':cs)
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\r':cs) = cont TokenConstant ('\r':cs)
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\t':cs) = cont TokenConstant ('\t':cs)
lexer cont ('t':'y':'p':'e':' ':cs) = cont TokenType cs
lexer cont ('t':'y':'p':'e':'\n':cs) = cont TokenType ('\n':cs)
lexer cont ('t':'y':'p':'e':'\r':cs) = cont TokenType ('\r':cs)
lexer cont ('t':'y':'p':'e':'\t':cs) = cont TokenType ('\t':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':' ':cs) = cont TokenFunction cs
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\n':cs) = cont TokenFunction ('\n':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\r':cs) = cont TokenFunction ('\r':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\t':cs) = cont TokenFunction ('\t':cs)
lexer cont ('e':'n':'c':'a':'p':' ':cs) = cont TokenEncap cs
lexer cont ('e':'n':'c':'a':'p':'\n':cs) = cont TokenEncap ('\n':cs)
lexer cont ('e':'n':'c':'a':'p':'\r':cs) = cont TokenEncap ('\r':cs)
lexer cont ('e':'n':'c':'a':'p':'\t':cs) = cont TokenEncap ('\t':cs)
lexer cont ('e':'n':'c':'a':'p':'(':cs) = cont TokenEncap ('(':cs)
lexer cont ('r':'e':'n':'a':'m':'e':' ':cs) = cont TokenRename cs
lexer cont ('r':'e':'n':'a':'m':'e':'\n':cs) = cont TokenRename ('\n':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'\r':cs) = cont TokenRename ('\r':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'\t':cs) = cont TokenRename ('\t':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'(':cs) = cont TokenRename ('(':cs)
lexer cont ('c':'o':'m':'m':' ':cs) = cont TokenComm cs
lexer cont ('c':'o':'m':'m':'\n':cs) = cont TokenComm ('\n':cs)
lexer cont ('c':'o':'m':'m':'\r':cs) = cont TokenComm ('\r':cs)
lexer cont ('c':'o':'m':'m':'\t':cs) = cont TokenComm ('\t':cs)
lexer cont (c:cs) 
      | isSpace c = lexer cont cs 
      | (\x -> isDigit x) c = lexNumber cont (c:cs)
      | (\x -> isAlpha x) c = lexString cont (c:cs)
lexer cont ('.':cs) = cont TokenSeq cs 
lexer cont ('|':'|':cs) = cont TokenIndependence cs
lexer cont ('|':'>':cs) = cont TokenElse cs 
lexer cont ('<':'|':cs) = cont TokenThen cs 
lexer cont ('&':cs) = cont TokenAnd cs 
lexer cont ('<':'=':cs) = cont TokenSmallerEq cs 
lexer cont ('>':'=':cs) = cont TokenGreaterEq cs 
lexer cont ('<':cs) = cont TokenSmaller cs 
lexer cont ('>':cs) = cont TokenGreater cs 
lexer cont ('(':cs) = cont TokenOB cs 
lexer cont ('|':cs) = cont TokenOr cs 
lexer cont (')':cs) = cont TokenCB cs 
lexer cont ('[':cs) = cont TokenOSB cs 
lexer cont (']':cs) = cont TokenCSB cs 
lexer cont ('{':cs) = cont TokenOA cs 
lexer cont ('}':cs) = cont TokenCA cs 
lexer cont ('+':'+':cs) = cont TokenPlusPlus cs
lexer cont ('-':'>':cs) = cont TokenProbDef cs
lexer cont (',':cs) = cont TokenComma cs
lexer cont ('+':cs) = cont TokenPlus cs
lexer cont ('-':cs) = cont TokenMinus cs
lexer cont ('!':'=':cs) = cont TokenNotEqual cs
lexer cont ('!':cs) = cont TokenNot cs
lexer cont ('/':cs) = cont TokenDivide cs
lexer cont ('*':cs) = cont TokenMultiply cs
lexer cont (':':'=':cs) = cont TokenBecomes cs
lexer cont (':':cs) = cont TokenColon cs
lexer cont ('=':'>':cs) = cont TokenImplies cs
lexer cont ('=':cs) = cont TokenEqual cs
lexer cont s = \line -> error("Error: Lexer could not deal with character \"" ++ [head s] ++ "\" on line " ++ show line)

lexString cont cs = cont (TokenString var) rest
  where
    (var,rest) = getString cs
lexNumber cont cs = cont (TokenString var) rest
  where
    (var,rest) = getNumber cs

getString [] = ("","")
getString ('.':'.':cs) = ("", '.':'.':cs)
getString (c:cs) | elem c "\n\t\r ',()&=*.<>+_{}:-!?;/|\"[]" = ("", c:cs)
                 | otherwise = (str,rest)
  where
    (s,r) = getString cs
    str   = c:s
    rest  = r

getNumber [] = ("","")
getNumber ('.':'.':cs) = ("", '.':'.':cs)
getNumber (c:cs) | elem c "\n\t\r ',()&=*<>+_{}:-!?;/|\"[]" = ("", c:cs)
                 | otherwise = (str,rest)
  where
    (s,r) = getNumber cs
    str   = c:s
    rest  = r
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "<built-in>" #-}
{-# LINE 1 "<command-line>" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
-- Id: GenericTemplate.hs,v 1.26 2005/01/14 14:47:22 simonmar Exp 

{-# LINE 30 "templates/GenericTemplate.hs" #-}








{-# LINE 51 "templates/GenericTemplate.hs" #-}

{-# LINE 61 "templates/GenericTemplate.hs" #-}

{-# LINE 70 "templates/GenericTemplate.hs" #-}

infixr 9 `HappyStk`
data HappyStk a = HappyStk a (HappyStk a)

-----------------------------------------------------------------------------
-- starting the parse

happyParse start_state = happyNewToken start_state notHappyAtAll notHappyAtAll

-----------------------------------------------------------------------------
-- Accepting the parse

-- If the current token is (1), it means we've just accepted a partial
-- parse (a %partial parser).  We must ignore the saved token on the top of
-- the stack in this case.
happyAccept (1) tk st sts (_ `HappyStk` ans `HappyStk` _) =
	happyReturn1 ans
happyAccept j tk st sts (HappyStk ans _) = 
	 (happyReturn1 ans)

-----------------------------------------------------------------------------
-- Arrays only: do the next action

{-# LINE 148 "templates/GenericTemplate.hs" #-}

-----------------------------------------------------------------------------
-- HappyState data type (not arrays)



newtype HappyState b c = HappyState
        (Int ->                    -- token number
         Int ->                    -- token number (yes, again)
         b ->                           -- token semantic value
         HappyState b c ->              -- current state
         [HappyState b c] ->            -- state stack
         c)



-----------------------------------------------------------------------------
-- Shifting a token

happyShift new_state (1) tk st sts stk@(x `HappyStk` _) =
     let (i) = (case x of { HappyErrorToken (i) -> i }) in
--     trace "shifting the error token" $
     new_state i i tk (HappyState (new_state)) ((st):(sts)) (stk)

happyShift new_state i tk st sts stk =
     happyNewToken new_state ((st):(sts)) ((HappyTerminal (tk))`HappyStk`stk)

-- happyReduce is specialised for the common cases.

happySpecReduce_0 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_0 nt fn j tk st@((HappyState (action))) sts stk
     = action nt j tk st ((st):(sts)) (fn `HappyStk` stk)

happySpecReduce_1 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_1 nt fn j tk _ sts@(((st@(HappyState (action))):(_))) (v1`HappyStk`stk')
     = let r = fn v1 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_2 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_2 nt fn j tk _ ((_):(sts@(((st@(HappyState (action))):(_))))) (v1`HappyStk`v2`HappyStk`stk')
     = let r = fn v1 v2 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_3 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_3 nt fn j tk _ ((_):(((_):(sts@(((st@(HappyState (action))):(_))))))) (v1`HappyStk`v2`HappyStk`v3`HappyStk`stk')
     = let r = fn v1 v2 v3 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happyReduce k i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyReduce k nt fn j tk st sts stk
     = case happyDrop (k - ((1) :: Int)) sts of
	 sts1@(((st1@(HappyState (action))):(_))) ->
        	let r = fn stk in  -- it doesn't hurt to always seq here...
       		happyDoSeq r (action nt j tk st1 sts1 r)

happyMonadReduce k nt fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyMonadReduce k nt fn j tk st sts stk =
        happyThen1 (fn stk tk) (\r -> action nt j tk st1 sts1 (r `HappyStk` drop_stk))
       where (sts1@(((st1@(HappyState (action))):(_)))) = happyDrop k ((st):(sts))
             drop_stk = happyDropStk k stk

happyMonad2Reduce k nt fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyMonad2Reduce k nt fn j tk st sts stk =
       happyThen1 (fn stk tk) (\r -> happyNewToken new_state sts1 (r `HappyStk` drop_stk))
       where (sts1@(((st1@(HappyState (action))):(_)))) = happyDrop k ((st):(sts))
             drop_stk = happyDropStk k stk





             new_state = action


happyDrop (0) l = l
happyDrop n ((_):(t)) = happyDrop (n - ((1) :: Int)) t

happyDropStk (0) l = l
happyDropStk n (x `HappyStk` xs) = happyDropStk (n - ((1)::Int)) xs

-----------------------------------------------------------------------------
-- Moving to a new state after a reduction

{-# LINE 246 "templates/GenericTemplate.hs" #-}
happyGoto action j tk st = action j j tk (HappyState action)


-----------------------------------------------------------------------------
-- Error recovery ((1) is the error token)

-- parse error if we are in recovery and we fail again
happyFail  (1) tk old_st _ stk =
--	trace "failing" $ 
    	happyError_ tk

{-  We don't need state discarding for our restricted implementation of
    "error".  In fact, it can cause some bogus parses, so I've disabled it
    for now --SDM

-- discard a state
happyFail  (1) tk old_st (((HappyState (action))):(sts)) 
						(saved_tok `HappyStk` _ `HappyStk` stk) =
--	trace ("discarding state, depth " ++ show (length stk))  $
	action (1) (1) tk (HappyState (action)) sts ((saved_tok`HappyStk`stk))
-}

-- Enter error recovery: generate an error token,
--                       save the old token and carry on.
happyFail  i tk (HappyState (action)) sts stk =
--      trace "entering error recovery" $
	action (1) (1) tk (HappyState (action)) sts ( (HappyErrorToken (i)) `HappyStk` stk)

-- Internal happy errors:

notHappyAtAll = error "Internal Happy error\n"

-----------------------------------------------------------------------------
-- Hack to get the typechecker to accept our action functions







-----------------------------------------------------------------------------
-- Seq-ing.  If the --strict flag is given, then Happy emits 
--	happySeq = happyDoSeq
-- otherwise it emits
-- 	happySeq = happyDontSeq

happyDoSeq, happyDontSeq :: a -> b -> b
happyDoSeq   a b = a `seq` b
happyDontSeq a b = b

-----------------------------------------------------------------------------
-- Don't inline any functions from the template.  GHC has a nasty habit
-- of deciding to inline happyGoto everywhere, which increases the size of
-- the generated parser quite a bit.

{-# LINE 310 "templates/GenericTemplate.hs" #-}
{-# NOINLINE happyShift #-}
{-# NOINLINE happySpecReduce_0 #-}
{-# NOINLINE happySpecReduce_1 #-}
{-# NOINLINE happySpecReduce_2 #-}
{-# NOINLINE happySpecReduce_3 #-}
{-# NOINLINE happyReduce #-}
{-# NOINLINE happyMonadReduce #-}
{-# NOINLINE happyGoto #-}
{-# NOINLINE happyFail #-}

-- end of Happy Template.
