------------
-- SUMELM --
------------

module Sumelm where

import Auxiliary
import Expressions
import LPPE 

-- This function removes unnecessary summations from an LPPE.
sumelm :: Specification -> Specification
sumelm ((LPPE name pars summands), initialState, dataspec) = (LPPE name pars newSummands, initialState, dataspec)
  where
    newSummands = map (\x -> sumelmSummand x (map fst (getLocalPars x))) summands

-- This function removes unncessary summations from a summand.
sumelmSummand :: Summand -> [Variable] -> Summand
sumelmSummand s [] = s
sumelmSummand (params, c, a, aps, probChoices, g) (var:vars) 
  | variableNotUsed = sumelmSummand (paramsNew, c, a, aps, probChoices, g) vars
  | onlyOneValue    = sumelmSummand (paramsNew, cNew, a, apsNew, probChoices, gNew) vars
  | otherwise       = sumelmSummand (params, c, a, aps, probChoices, g) vars
  where
    paramsNew                = [(v,t) | (v,t) <- params, not (v == var)]
    variableNotUsed          = not(variableInExpression var c) && 
                               not(variableInExpressions var aps) &&
                               not(variableInExpressions var g)
    value                    = possibleValueForConditionToBeTrue var c
    onlyOneValue             = value /= Undecided
    UniqueValue uniqueChoice = value
    cNew                     = substituteInExpression [(var, uniqueChoice)] c
    apsNew                   = map (substituteInExpression [(var, uniqueChoice)]) aps
    gNew                     = if (elem var (map fst3 probChoices)) then g else (map (substituteInExpression [(var, uniqueChoice)]) g)