module ToAUT where

import ToPA
import LPPE
import Confluence
import DataSpec
import Ratio

-------------------------------
-- Translation to AUT format --
-------------------------------

data OutputString = OutputString [Char]
instance Show OutputString where
   show (OutputString s) = s


toAUTNonProb :: Bool -> Specification -> Bool -> ConfluentSummands -> OutputString
toAUTNonProb ignorecycles spec cadp confl = OutputString ("des (0," ++ show (length transitions2) ++ "," ++ show (length states) ++ ") \n" ++ (printTransitionsNonProb cadp transitions2))
  where
    (statespace,initial,_) = getStateSpace ignorecycles spec confl False
    states                 = fst statespace
    transitions            = snd statespace
    transitions2           = createTransitionsNonProb transitions

createTransitionsNonProb :: [(Int, EdgeLabel, [(Probability, Int)])] -> [(Int, EdgeLabel, Int)]
createTransitionsNonProb []                              = []
createTransitionsNonProb ((from, label, []):trans)       = createTransitionsNonProb trans
createTransitionsNonProb ((from, label, (to:tos)):trans) = (from, label, snd to):rest
  where
    rest = createTransitionsNonProb ((from, label, tos):trans)

printTransitionsNonProb cadp [] = []
printTransitionsNonProb cadp ((from, label, to):ts) = "(" ++ show from ++ ",\"" ++ (if cadp then (if label == "tau" then "i" else label) else label) ++ "\"," ++ show to ++ ")\n" ++ (printTransitionsNonProb cadp ts)


toAUTProb :: Bool -> Specification -> Bool -> ConfluentSummands -> OutputString
toAUTProb ignorecycles spec cadp confl = OutputString ("des (0," ++ show (length transitions + length transitions2) ++ "," ++ show (length states + length transitions) ++ ") \n" ++ (printTransitionsProb cadp transitions (length states)))
    where
     (statespace,initial,_) = getStateSpace ignorecycles spec confl False
     states                 = fst statespace
     transitions            = snd statespace
     transitions2           = createTransitionsNonProb transitions

printTransitionsProb :: Bool -> [(Int, EdgeLabel, [(Probability, Int)])] -> Int -> [Char] 
printTransitionsProb cadp [] i                     = []
printTransitionsProb cadp ((from, label, []):ts) i = "(" ++ show from ++ ",\"" ++ (if cadp then (if label == "tau" then "i" else label) else label) ++ "\"," ++ show i ++ ")\n" ++ (printTransitionsProb cadp ts (i+1))
printTransitionsProb cadp ((from, label, ((prob,to):tos)):ts) i = "(" ++ show i ++ ",\"prob " ++ (if cadp then printFraction prob else prob) ++ "\"," ++ show to ++ ")\n" ++ (printTransitionsProb cadp ((from, label, tos):ts) i)
  where
    changeFraction = \x -> (toInteger (numerator x)) % (toInteger (denominator x))
    printFraction  = \x -> show (fromRational (changeFraction (getFraction prob)))