-- TODO: checken of 'changed' wel klopt zo. Misschien rekening houden met nondet/prob sums.

module Main where

import Auxiliary
import InputParser
import LPPE
import ToPA
import ToAUT
import ToTrans
import Expressions
import ToPRISM
import System.IO
import System (getArgs)
import Simplify
import Confluence
import Sumelm
import Constelm
import Parelm
import DeadVariable
import Control.Monad
import FromPRISM

main = do
  args <- getArgs
  input <- hGetContents stdin
  
  -- Parsing the command-line parameters.
  let includingConfluence  = elem "-conf" args
  let includingStateGraph  = elem "-dead" args
  let verbose              = elem "-verbose" args
  let printLPPE            = elem "-lppe" args
  let stateSpaceSize       = elem "-size" args
  let prism                = elem "-prism" args
  let fromprism            = elem "-fromprism" args
  let noprobs              = elem "-noprob" args
  let visited              = elem "-visited" args
  let printAUTfile         = elem "-aut" args
  let cadp                 = elem "-cadp" args
  let states               = elem "-statespace" args
  let prcrl                = elem "-prcrl" args
  let checkuntil           = elem "-checkuntil" args
  let trans                = elem "-trans" args
  let includingParelm      = elem "-parelm" args
  let removecycles         = elem "-removecycles" args
  let ignorecycles         = elem "-ignorecycles" args
  let pastyle              = elem "-PAstyle" args || prcrl
  let strong               = elem "-strong" args
  let sourcefiles          = [x | ('-':'s':'o':'u':'r':'c':'e':'=':x) <- args]
  let constantdefs         = [x | ('-':'c':'o':'n':'s':'t':'a':'n':'t':'-':x) <- args]
  let flags      = ["-conf", "-dead", "-fromprism", "-checkuntil", "-cadp", "-verbose", "-trans", "-lppe", "-size", "-prism", "-aut", "-visited", "-noprob",
                    "-ignorecycles", "-statespace", "-prcrl", "-parelm", "-removecycles", "-PAstyle", "-strong"]
  let wrongFlags = [x | x <- args, not(elem x flags || take 8 x == "-source=" || take 10 x == "-constant-")]

  when (length wrongFlags > 0) $ putStrLn ("Warning: flags " ++ show wrongFlags ++ " not recognised.")

  -- Parsing the constants.
  let constants               = [(takeWhile (/= '=') constant, Variable (drop 1 (dropWhile (/= '=') constant))) | constant <- constantdefs]

  -- Parsing the input, and doing all the relevant transformations.
  let prismTranslation         = if fromprism then fromPrism else id
  let (basicSpec, actiontypes, untilformula) = (parseInput fromprism constants . prismTranslation) input
  let standardTransformations  = simplify . sumelm . simplify . constelm . sumelm
  let transformations          = (if includingStateGraph then (constelm . deadVariableReduction) else 
	                                     if includingParelm then parelm else id) . standardTransformations
  let specification            = transformations basicSpec
  --putStrLn (prismTranslation input)

  -- Outputting information about the procedure.
  when (verbose && (length sourcefiles) == 1) $ putStrLn ("Specification that was analysed: " ++ (sourcefiles!!0))
  when (verbose) $ putStrLn ("Confluence reduction: " ++ show includingConfluence)
  when (verbose) $ putStrLn ("Dead variable reduction: " ++ show includingStateGraph ++ "\n")
 
  -- Showing the LPPE and the number of parameters / summands.
  when (prcrl)                   $ print (LPPEShowPRCRL specification)
  when (printLPPE && not(prcrl)) $ print (LPPEShow specification pastyle) 
  when (printLPPE && verbose)    $ putStrLn ("Number of parameters: " ++ show (length (getLPPEPars (getLPPE specification))))
  when (printLPPE && verbose)    $ putStrLn ("Number of summands:   " ++ show (length (getSummands (getLPPE specification))))
 
  -- Detecting confluence.
  let confluents_ = if includingConfluence then confluentSummands specification strong else []
  let confluents  = if not(removecycles) then confluents_ else removeConfluentCycles specification confluents_ 
  when (includingConfluence && verbose) $ putStrLn ("Confluent summands:   " ++ show confluents)

  -- Computing the actual state space.
  --let statespacesize    = fst (getStatesAndTransitionsConfluence ignorecycles specification confluents False)
  let statespacesize = getStatesAndTransitionsConfluence ignorecycles specification confluents True

  -- Showing information about the state space.
  when stateSpaceSize $ putStrLn ("Number of states: " ++ show (fst (fst statespacesize)))
  when stateSpaceSize $ putStrLn ("Number of transitions: " ++ show (snd (fst statespacesize)))
  when visited        $ putStrLn ("Number of states visited: " ++ show (fst (snd statespacesize)))
  when visited        $ putStrLn ("Number of transitions visited: " ++ show (snd (snd statespacesize)))

  -- Printing the state space to an AUT file (either probabilistically or non-probabilistically).
  when (printAUTfile && noprobs) $ print (toAUTNonProb ignorecycles specification cadp confluents)
  when (printAUTfile && (not(noprobs))) $ print (toAUTProb ignorecycles specification cadp confluents)

  -- Printing the state space to PRISM's trans format.
  when trans               $ putStrLn (toTrans ignorecycles specification confluents)

  -- Printing all the actual states and transitions.
  when states $ print (fst3 (getStateSpace ignorecycles specification confluents False)) 
  
  -- Outputting to PRISM.
  let specificationPRISM = if includingConfluence then doSyntacticConfluence specification confluents else specification
  when prism $ print (toPRISM actiontypes checkuntil untilformula specificationPRISM)